<?php

if (!defined('WHMCS')) {
    exit('This file cannot be accessed directly');
}

use WHMCS\Carbon;
use WHMCS\Domain\Registrar\Domain;
use WHMCS\Domain\TopLevel\ImportItem;
use WHMCS\Results\ResultsList;

require_once 'classEuroDNS.php';

function EuroDNS_getConfigArray()
{
    return [
        'Username' => ['Type' => 'text', 'Size' => '20', 'Description' => 'Enter your username here'],
        'Password' => ['Type' => 'password', 'Size' => '20', 'Description' => 'Enter your password here'],
        'TestMode' => ['Type' => 'yesno'],
    ];
}

function checkIfDomainIsActive($params)
{
    $command = 'GetClientsDomains';
    $postData = [
        'clientid' => $params['userid'],
        'stats' => true,
    ];

    $results = localAPI($command, $postData);

    foreach ($results['domains']['domain'] as $key => $domain) {
        if (idn_to_ascii($params['sld']).'.'.idn_to_ascii($params['tld']) === $domain['domainname']) {
            if ('Transferred Away' === $domain['status'] || 'Cancelled' === $domain['status']) {
                return false;
            }

            return true;
        }
    }

    return false;
}

function EuroDNS_GetDomainInformation($params)
{
    if (!checkIfDomainIsActive($params)) {
        return new Domain();
    }
    $EuroDNS = new EuroDNS([
        'Username' => $params['Username'],
        'Password' => $params['Password'],
        'TestMode' => $params['TestMode'],
    ]);

    $data = [
        'tld' => idn_to_ascii($params['tld']),
        'sld' => idn_to_ascii($params['sld']),
    ];

    $info = $EuroDNS->getInfo($data);

    if (!$info) {
        if ('9999' === $EuroDNS->getErrorCode()) {
            throw new Exception('Wrong username and/or wrong password specified.');
        }

        throw new Exception('No Information about this Domain is available from Registrar. Please check Registrar or register Domain.');
    }

    /// $xml = simplexml_import_dom($info);
    /// if (!$xml) {
    ///     echo 'Error while parsing the document';
    ///     exit;
    /// }

    $validation = (string) $info->xpath('/response/extension/eurodns:contact/eurodns:validated')[0];
    $infoTLD = $EuroDNS->infoTld(idn_to_ascii($params['tld']));
    //$val = $EuroDNS->getCachedInfo(array("tld",$params["tld"],"needContactValidation"),true);
    $idProdStatus = false;

    $val = '';
    if ($infoTLD['needContactValidation']) {
        $val = $infoTLD['needContactValidation'];
    }

    /// update id protection for tld automatically. Maybe need to be refactored
    //$tldidproneed = $EuroDNS->getCachedInfo(array("tld",$params["tld"],"whoisPrivacy"));
    $tldidproneed = $infoTLD['whoisPrivacy'];

    if ('no' === strtolower($tldidproneed)) {
        $command = 'CreateOrUpdateTLD';
        $postData = [
            'extension' => '.'.idn_to_ascii($params['tld']),
            'id_protection' => false,
            //'epp_required' => true,
        ];

        $results = localAPI($command, $postData);
        logModuleCall('EuroDNS', 'update TLD', 'set id/whois protection to false, since EuroDNS not allow for thie TLD'.idn_to_ascii($params['tld']), '', null, null);
    } else {
        $idPr = (string) $info->xpath('/response/extension/extension:service/service:domainprivacy')[0];
        if ($idPr && 'yes' === strtolower($idPr)) {
            $idProdStatus = true;
        }
    }

    if ($validation) {
        if ('yes' === strtolower($val)) {
            $expireDate = (string) $info->xpath('/response/extension/eurodns:contact/eurodns:validated')[0]->attributes()->endValidityDate;
            $sdccp = false;
            if (empty($expireDate) || !$expireDate) {
                $expireDate = null;
            } else {
                $expireDate = Carbon::createFromFormat('Y-m-d', $expireDate);
            }

            if ('no' === strtolower($validation)) {
                $sdccp = true;
            }

            return (new Domain())
                ->setIsIrtpEnabled(true)
                ->setIrtpOptOutStatus(false)
                ->setIdProtectionStatus($idProdStatus)
                ->setDomainContactChangePending($sdccp)
                ->setDomainContactChangeExpiryDate($expireDate)
                ->setIrtpVerificationTriggerFields(
                    [
                        'Registrant' => [
                            'First Name',
                            'Last Name',
                            'Organization Name',
                            'Email',
                        ],
                    ]
                )
            ;
        }
    }

    return (new Domain())->setIsIrtpEnabled(false)->setIdProtectionStatus($idProdStatus);
}

function EuroDNS_GetNameservers($params)
{
    $EuroDNS = new EuroDNS([
        'Username' => $params['Username'],
        'Password' => $params['Password'],
        'TestMode' => $params['TestMode'],
    ]);

    $data = [
        'tld' => idn_to_ascii($params['tld']),
        'sld' => idn_to_ascii($params['sld']),
    ];

    $ns = $EuroDNS->getInfo($data, 'ns');
    if (!$ns) {
        return $return['error'] = 'No username and/or Password set, please add your account under Setup -> Product/Services -> Domain Registrars -> EuroDNS';
    }
    if ($error = $EuroDNS->getError()) {
        $return['error'] = $error;
    }

    if ($EuroDNS->startsWith('ERROR', $ns)) {
        $return['error'] = 'Domain could not be found from EuroDNS, please see Module log or check if domain exist';

        return $return;
    }

    if (is_a($ns, 'SimpleXMLElement') && ((string) $ns->result->attributes()[0]) === '5111') {
        $command = 'GetClientsDomains';
        $postData = [
            'clientid' => $params['userid'],
            'stats' => true,
        ];

        $results = localAPI($command, $postData);

        foreach ($results['domains']['domain'] as $key => $domain) {
            if (idn_to_ascii($params['sld']).'.'.idn_to_ascii($params['tld']) === $domain['domainname']) {
                if ('Cancelled' !== $domain['status'] && 'Transferred Away' !== $domain['status']) {
                    $return['error'] = 'Registrar Domain info","Domain not exist, status will change to Transferred Away on next cron job run';
                }
            }
        }

        return $return;
    }

    if (!empty($EuroDNS->getPendingMessage())) {
        $return['error'] = 'here is some pending actions from your registrar. The information that appears on the page might be not up to date. Current action: '.$EuroDNS->getPendingMessage().' - Please wait 5-10 min and refresh the page';
    }

    if (empty($ns)) {
        $return['error'] = 'NS could be not found for the domain';

        return $return;
    }

    $i = 1;
    foreach ($ns as $n) {
        if ($n) {
            $return['ns'.$i++] = $n;
        }
    }
    logModuleCall('EuroDNS', 'GetNameservers', $EuroDNS->getRequest(), $EuroDNS->getResponse(), null, null);

    return $return;
}

function EuroDNS_SaveNameservers($params)
{
    $EuroDNS = new EuroDNS([
        'Username' => $params['Username'],
        'Password' => $params['Password'],
        'TestMode' => $params['TestMode'],
    ]);

    $data = [
        'tld' => idn_to_ascii($params['tld']),
        'sld' => idn_to_ascii($params['sld']),
        'ns' => [$params['ns1'], $params['ns2'], $params['ns3'], $params['ns4'], $params['ns5']],
    ];

    $EuroDNS->updateNs($data);
    logModuleCall('EuroDNS', 'SaveNameservers', $EuroDNS->getRequest(), $EuroDNS->getResponse(), null, null);

    // Has  error ?
    if ($e = $EuroDNS->getError()) {
        $values['error'] = $e;
    }

    return $values;
}

function EuroDNS_GetRegistrarLock($params)
{
    $EuroDNS = new EuroDNS([
        'Username' => $params['Username'],
        'Password' => $params['Password'],
        'TestMode' => $params['TestMode'],
    ]);

    $data = [
        'tld' => idn_to_ascii($params['tld']),
        'sld' => idn_to_ascii($params['sld']),
    ];

    $infoTLD = $EuroDNS->infoTld(idn_to_ascii($params['tld']));
    //$val = $EuroDNS->getCachedInfo(array("tld",$params["tld"],"lockingEnabled"),true);
    $val = $infoTLD['lockingEnabled'];

    if ('no' === strtolower($val)) {
        return;
    }
    $info = $EuroDNS->getInfo($data);

    if (!$info) {
        return;
    }
    $status = (string) $info->xpath('/response/resData/domain:lockStatus')[0];

    if ('unlocked' === $status || 'pending lock' === $status) {
        return 'unlocked';
    }

    return 'locked';
}

function EuroDNS_SaveRegistrarLock($params)
{
    $EuroDNS = new EuroDNS([
        'Username' => $params['Username'],
        'Password' => $params['Password'],
        'TestMode' => $params['TestMode'],
    ]);

    $data = [
        'tld' => idn_to_ascii($params['tld']),
        'sld' => idn_to_ascii($params['sld']),
    ];

    if ('locked' == $params['lockenabled']) {
        $EuroDNS->Lock($data);
        logModuleCall('EuroDNS', 'SaveRegistrarLock', $EuroDNS->getRequest(), $EuroDNS->getResponse().$EuroDNS->getError(), null, null);
    } else {
        $EuroDNS->UnLock($data);
        logModuleCall('EuroDNS', 'SaveRegistrarUnLock', $EuroDNS->getRequest(), $EuroDNS->getResponse().$EuroDNS->getError(), null, null);
    }

    if ($e = $EuroDNS->getError()) {
        return [
            'error' => $e,
        ];
    }

    return [
        'success' => 'success',
    ];
}

function EuroDNS_RegisterDomain($params)
{
    $EuroDNS = new EuroDNS([
        'Username' => $params['Username'],
        'Password' => $params['Password'],
        'TestMode' => $params['TestMode'],
    ]);

    $data = [
        'tld' => idn_to_ascii($params['tld']),
        'sld' => idn_to_ascii($params['sld']),
        'regperiod' => $params['regperiod'],
        'renewal' => 'autoexpire',
        'ns' => [$params['ns1'], $params['ns2'], $params['ns3'], $params['ns4'], $params['ns5']],
        // Client datails
        'firstname' => $params['firstname'],
        'lastname' => $params['lastname'],
        'address1' => $params['address1'],
        'company' => $params['companyname'],
        'address2' => $params['address2'],
        'city' => $params['city'],
        'postcode' => $params['postcode'],
        'country_code' => $params['country'],
        'email' => $params['email'],
        'phonenumber' => $params['phonenumber'],
        'fax' => null,
        'state' => $params['state'],
        // Admin datails
        'adminfirstname' => $params['adminfirstname'],
        'adminlastname' => $params['adminlastname'],
        'admincompany' => $params['admincompanyname'],
        'adminaddress1' => $params['adminaddress1'],
        'adminaddress2' => $params['adminaddress2'],
        'admincity' => $params['admincity'],
        'adminpostcode' => $params['adminpostcode'],
        'admincountry_code' => $params['admincountry'],
        'adminemail' => $params['adminemail'],
        'adminphonenumber' => $params['adminphonenumber'],
        'adminfax' => null,
        // Tech datails
        'techfirstname' => $params['adminfirstname'],
        'techlastname' => $params['adminlastname'],
        'techcompany' => $params['admincompanyname'],
        'techaddress1' => $params['adminaddress1'],
        'techaddress2' => $params['adminaddress2'],
        'techcity' => $params['admincity'],
        'techpostcode' => $params['adminpostcode'],
        'techcountry_code' => $params['admincountry'],
        'techemail' => $params['adminemail'],
        'techphonenumber' => $params['adminphonenumber'],
        'techfax' => null,
        //Biling datails
        'billingfirstname' => $params['adminfirstname'],
        'billinglastname' => $params['adminlastname'],
        'billingcompany' => $params['admincompanyname'],
        'billingaddress1' => $params['adminaddress1'],
        'billingaddress2' => $params['adminaddress2'],
        'billingcity' => $params['admincity'],
        'billingpostcode' => $params['adminpostcode'],
        'billingcountry_code' => $params['admincountry'],
        'billingemail' => $params['adminemail'],
        'billingphonenumber' => $params['adminphonenumber'],
        'billinfax' => null,
        'additionalfields' => $params['additionalfields'],
    ];

    // Register Domain

    $EuroDNS->Create($data, $params['idprotection']);
    logModuleCall('EuroDNS', 'RegisterDomain', $EuroDNS->getRequest(), $EuroDNS->getResponse(), null, null);

    if ($params['idprotection']) {
        $data = [];
        $data['protectenable'] = $params['idprotection'];
        $EuroDNS->toggleIDProtection($data);
    }

    if ($e = $EuroDNS->getError()) {
        $values['error'] = $e;
    }

    return $values;
}

function EuroDNS_TransferDomain($params)
{
    $EuroDNS = new EuroDNS([
        'Username' => $params['Username'],
        'Password' => $params['Password'],
        'TestMode' => $params['TestMode'],
    ]);

    $data = [
        'tld' => idn_to_ascii($params['tld']),
        'sld' => idn_to_ascii($params['sld']),
        'transfersecret' => $params['transfersecret'],
        'ns' => [$params['ns1'], $params['ns2'], $params['ns3'], $params['ns4'], $params['ns5']],
        // Client datails
        'firstname' => $params['firstname'],
        'lastname' => $params['lastname'],
        'address1' => $params['address1'],
        'company' => $params['companyname'],
        'address2' => $params['address2'],
        'city' => $params['city'],
        'postcode' => $params['postcode'],
        'country_code' => $params['country'],
        'email' => $params['email'],
        'phonenumber' => $params['phonenumber'],
        'fax' => null,
        // Admin datails
        'adminfirstname' => $params['adminfirstname'],
        'adminlastname' => $params['adminlastname'],
        'admincompany' => $params['admincompanyname'],
        'adminaddress1' => $params['adminaddress1'],
        'adminaddress2' => $params['adminaddress2'],
        'admincity' => $params['admincity'],
        'adminpostcode' => $params['adminpostcode'],
        'admincountry_code' => $params['admincountry'],
        'adminemail' => $params['adminemail'],
        'adminphonenumber' => $params['phonenumber'],
        'adminfax' => null,
        // Tech datails
        'techfirstname' => $params['adminfirstname'],
        'techlastname' => $params['adminlastname'],
        'techcompany' => $params['admincompanyname'],
        'techaddress1' => $params['adminaddress1'],
        'techaddress2' => $params['adminaddress2'],
        'techcity' => $params['admincity'],
        'techpostcode' => $params['adminpostcode'],
        'techcountry_code' => $params['admincountry'],
        'techemail' => $params['adminemail'],
        'techphonenumber' => $params['phonenumber'],
        'techfax' => null,
        //Biling datails
        'billingfirstname' => $params['adminfirstname'],
        'billinglastname' => $params['adminlastname'],
        'billingcompany' => $params['admincompanyname'],
        'billingaddress1' => $params['adminaddress1'],
        'billingaddress2' => $params['adminaddress2'],
        'billingcity' => $params['admincity'],
        'billingpostcode' => $params['adminpostcode'],
        'billingcountry_code' => $params['admincountry'],
        'billingemail' => $params['adminemail'],
        'billingphonenumber' => $params['phonenumber'],
        'billinfax' => null,
        'additionalfields' => $params['additionalfields'],
    ];

    $EuroDNS->Transfer($data);
    logModuleCall('EuroDNS', 'TransferDomain', $EuroDNS->getRequest(), $EuroDNS->getResponse(), null, null);

    if ($e = $EuroDNS->getError()) {
        $values['error'] = $e;
    }

    return $values;
}

function EuroDNS_RenewDomain($params)
{
    $EuroDNS = new EuroDNS([
        'Username' => $params['Username'],
        'Password' => $params['Password'],
        'TestMode' => $params['TestMode'],
    ]);

    $data = [
        'sld' => idn_to_ascii($params['sld']),
        'tld' => idn_to_ascii($params['tld']),
        'regperiod' => $params['regperiod'],
        'additionalfields' => $params['additionalfields'],
    ];

    $info = $EuroDNS->getInfo($data);

    if (!$info) {
        return;
    }

    if (((string) $info->result->attributes()[0]) === '5111') {
        logModuleCall('EuroDNS', 'Renew Domain', $EuroDNS->getRequest(), $EuroDNS->getResponse(), null, null);
        $values['error'] = 'Domain could not be found from EuroDNS, please see Module log or check if domain exist';

        return $values;
    }

    $time = strtotime((string) $info->xpath('/response/resData/domain:expDate')[0]);
    $now = strtotime(date('Y-m-d', time()));

    if ($now > $time) {
        $EuroDNS->Reactivate($data);
        logModuleCall('EuroDNS', 'Reactivate Domain', $EuroDNS->getRequest(), $EuroDNS->getResponse(), null, null);
    } else {
        $EuroDNS->Renew($data);
        logModuleCall('EuroDNS', 'Renew Domain', $EuroDNS->getRequest(), $EuroDNS->getResponse(), null, null);
    }

    if ($e = $EuroDNS->getError()) {
        return ['error' => $e];
    }

    return ['success' => true];
}

function EuroDNS_GetContactDetails($params)
{
    $EuroDNS = new EuroDNS([
        'Username' => $params['Username'],
        'Password' => $params['Password'],
        'TestMode' => $params['TestMode'],
    ]);

    $data = [
        'tld' => idn_to_ascii($params['tld']),
        'sld' => idn_to_ascii($params['sld']),
    ];

    $values = $EuroDNS->getContact($data);
    if (!$values) {
        return $return['error'] = 'No username and/or Password set, please add your account under Setup -> Product/Services -> Domain Registrars -> EuroDNS';
    }
    if ($e = $EuroDNS->getError()) {
        $values['error'] = $e;
    }

    logModuleCall('EuroDNS', 'GetContactDetails', $EuroDNS->getRequest(), $EuroDNS->getResponse(), null, null);

    return $values;
}

function EuroDNS_SaveContactDetails($params)
{
    $EuroDNS = new EuroDNS([
        'Username' => $params['Username'],
        'Password' => $params['Password'],
        'TestMode' => $params['TestMode'],
    ]);

    $EuroDNS->UpdateContactDetails($params['contactdetails']['Registrant']);
    $EuroDNS->UpdateContactDetails($params['contactdetails']['Admin']);
    $EuroDNS->UpdateContactDetails($params['contactdetails']['Tech']);
    $EuroDNS->UpdateContactDetails($params['contactdetails']['Billing']);

    $data = [
        'tld' => idn_to_ascii($params['tld']),
        'sld' => idn_to_ascii($params['sld']),
        // Client datails
        'firstname' => $params['contactdetails']['Registrant']['First Name'],
        'lastname' => $params['contactdetails']['Registrant']['Last Name'],
        'address1' => $params['contactdetails']['Registrant']['address1'],
        'company' => $params['contactdetails']['Registrant']['company'],
        'address2' => null,
        'city' => $params['contactdetails']['Registrant']['city'],
        'postcode' => $params['contactdetails']['Registrant']['postcode'],
        'country_code' => $params['contactdetails']['Registrant']['country'],
        'email' => $params['contactdetails']['Registrant']['Email'],
        'phonenumber' => $params['contactdetails']['Registrant']['phonenumber'],
        'fax' => $params['contactdetails']['Registrant']['Fax'],
        // Admin datails
        'adminfirstname' => $params['contactdetails']['Admin']['First Name'],
        'adminlastname' => $params['contactdetails']['Admin']['Last Name'],
        'admincompany' => $params['contactdetails']['Admin']['company'],
        'adminaddress1' => $params['contactdetails']['Admin']['address1'],
        'adminaddress2' => null,
        'admincity' => $params['contactdetails']['Admin']['city'],
        'adminpostcode' => $params['contactdetails']['Admin']['postcode'],
        'admincountry_code' => $params['contactdetails']['Admin']['country'],
        'adminemail' => $params['contactdetails']['Admin']['Email'],
        'adminphonenumber' => $params['contactdetails']['Admin']['phonenumber'],
        'adminfax' => $params['contactdetails']['Admin']['Fax'],
        // Tech datails
        'techfirstname' => $params['contactdetails']['Tech']['First Name'],
        'techlastname' => $params['contactdetails']['Tech']['Last Name'],
        'techcompany' => $params['contactdetails']['Tech']['company'],
        'techaddress1' => $params['contactdetails']['Tech']['address1'],
        'techaddress2' => null,
        'techcity' => $params['contactdetails']['Tech']['city'],
        'techpostcode' => $params['contactdetails']['Tech']['postcode'],
        'techcountry_code' => $params['contactdetails']['Tech']['country'],
        'techemail' => $params['contactdetails']['Tech']['Email'],
        'techphonenumber' => $params['contactdetails']['Tech']['phonenumber'],
        'techfax' => $params['contactdetails']['Tech']['Fax'],
        //Biling datails
        'billingfirstname' => $params['contactdetails']['Billing']['First Name'],
        'billinglastname' => $params['contactdetails']['Billing']['Last Name'],
        'billingcompany' => $params['contactdetails']['Billing']['company'],
        'billingaddress1' => $params['contactdetails']['Billing']['address1'],
        'billingaddress2' => null,
        'billingcity' => $params['contactdetails']['Billing']['city'],
        'billingpostcode' => $params['contactdetails']['Billing']['postcode'],
        'billingcountry_code' => $params['contactdetails']['Billing']['country'],
        'billingemail' => $params['contactdetails']['Billing']['Email'],
        'billingphonenumber' => $params['contactdetails']['Billing']['phonenumber'],
        'billinfax' => $params['contactdetails']['Billing']['Fax'],
        'additionalfields' => $params['additionalfields'],
    ];

    $EuroDNS->updateContact($data);
    logModuleCall('EuroDNS', 'SaveContactDetails', $EuroDNS->getRequest(), $EuroDNS->getResponse(), null, null);

    if ($e = $EuroDNS->getError()) {
        $values['error'] = $e;
    }

    logModuleCall('EuroDNS', 'GetContactDetails', $EuroDNS->getRequest(), $EuroDNS->getResponse(), null, null);

    return $values;
}

/// get all DNS data
function EuroDNS_GetDNS($params)
{
    global $EuroDNS;

    $EuroDNS = new EuroDNS([
        'Username' => $params['Username'],
        'Password' => $params['Password'],
        'TestMode' => $params['TestMode'],
    ]);

    $result = $EuroDNS->zoneInfo($params);
    if (!$result) {
        return $return['error'] = 'No username and/or Password set, please add your account under Setup -> Product/Services -> Domain Registrars -> EuroDNS';
    }
    logModuleCall('EuroDNS', 'GetDNS', $EuroDNS->getRequest(), $EuroDNS->getResponse(), null, null);
    $xml = simplexml_load_string($EuroDNS->validateResponse($result));

    if (!$xml) {
        return ['error' => 'Server response error'];
    }

    if (1000 != $xml->result->attributes()) {
        return ['error' => $xml->result->msg];
    }

    foreach ($xml->resData as $record) {
        foreach ($record->children('http://www.eurodns.com/zone') as $value2) {
            $i = 0;
            foreach ($value2->record as $value3) {
                foreach ($value3->children('http://www.eurodns.com/record') as $key => $value4) {
                    $dns[$i][$key] = (string) $value4;
                }

                foreach ($value3->attributes() as $key => $value) {
                    $dns[$i][$key] = (string) $value[0];
                }
                ++$i;
            }
        }
    }

    $return_array = [];
    if (!$dns) {
        return;
    }

    foreach ($dns as $element) {
        $prio = 'N/A';

        if ('MX' == $element['type']) {
            $prio = $element['mx_priority'];
        } elseif ('SRV' == $element['type']) {
            $prio = $element['priority'];
        }

        $return_array[] = [
            'hostname' => $element['host'],
            'address' => $element['data'],
            'type' => $element['type'],
            'recid' => $element['id'],
            'ttl' => $element['ttl'],
            'priority' => $prio,
            'weight' => (('SRV' == $element['type']) ? $element['weight'] : ''),
            'port' => (('SRV' == $element['type']) ? $element['port'] : ''),
        ];
    }

    return $return_array;
}

///Update DNS to EuroDNS
function EuroDNS_SaveDNS($params)
{
    $APIDNS = EuroDNS_GetDNS($params);
    $WHMCSDNS = $params['dnsrecords'];
    $error = false;
    $postData = $_POST;
    $rmids = $postData['dnsrecordremove'];
    global $EuroDNS;
    $dns_data = [];
    $mxCounter = 0;
    $srvCounter = 0;
    $recordCounter = 0;

    foreach ($APIDNS as $element) {
        if ('SOA' !== $element['type']) {
            $APIDNS_reformed[$element['recid']] = $element;
        }
    }

    foreach ($WHMCSDNS as $key => $record) {
        $record['recid'] = $postData['dnsrecordrecid'][$recordCounter];
        $record['ttl'] = $postData['dnsrecordttl'][$recordCounter];
        ++$recordCounter;
        if (empty($record['hostname']) || empty($record['address'])) {
            continue;
        }
        if ('N/A' == $record['priority']) {
            unset($record['priority']);
        }

        if ('MX' == $record['type']) {
            $priority = $postData['dnsrecordmxpriority'][$mxCounter];
            ++$mxCounter;
        } elseif ('SRV' == $record['type']) {
            $priority = $postData['dnsrecordsrvpriority'][$srvCounter];
            $record['address'] = $record['address'].','.$postData['dnsrecordsrvweight'][$srvCounter].','.$postData['dnsrecordsrvport'][$srvCounter];
            ++$srvCounter;
        }

        $dns = [
            'host' => $record['hostname'],
            'data' => $record['address'],
            'type' => $record['type'],
            'recid' => $record['recid'],
            'ttl' => $record['ttl'],
        ];

        if ('MX' == $record['type']) {
            $dns['mx_priority'] = $priority;
        }

        if ('SRV' == $record['type']) {
            $dns['priority'] = $priority;

            $data = explode(',', $record['address']);
            array_walk($data, function (&$item, $key) {
                $item = trim($item);
            });

            $rmids = $postData['dnsrecordremove'];

            $dns['data'] = $data[0];
            $dns['weight'] = $data[1];
            $dns['port'] = $data[2];

            array_walk($dns, function (&$item, $key) use (&$error) {
                if (empty($item) && 'recid' != $key) {
                    $error = true;
                }
            });

            if ($error) {
                return ['error' => 'Error with record SRV. Please provide data to Address, weight, port: -> host:'.$record['hostname'].' | data:'.$record['address'].' | prio:'.$dns['mx_priority']];
            }
        }

        if (!empty($record['recid']) && '' != $record['recid']) {
            if (null !== $rmids && in_array($dns['recid'], $rmids)) {
                //# REMOVING ##
                $dns_data['remove'][]['recid'] = $dns['recid'];
            } else {
                //# UPDATING ##
                $diff = array_diff($dns, $APIDNS_reformed[$dns['recid']]);
                if (!empty($diff)) {
                    $diff['recid'] = $dns['recid'];
                    $dns_data['change'][] = $dns;
                }
            }
        } elseif ('' != $dns['host'] && '' != $dns['data']) {
            //# ADDING ##
            $dns_data['add'][] = $dns;
        }
    }

    $data = [
        'tld' => idn_to_ascii($params['tld']),
        'sld' => idn_to_ascii($params['sld']),
        'dns' => $dns_data,
    ];

    $result = $EuroDNS->updateDNS($data);
    logModuleCall('EuroDNS', 'UpdateDNS', $EuroDNS->getRequest(), $EuroDNS->getResponse(), null, null);

    $isSRV = false;

    if ('success' == $result) {
        return 'success';
    }

    return ['error' => $EuroDNS->getError()];
}

function EuroDNS_TransferSync($params)
{
    $return = [];

    logModuleCall('EuroDNS', 'TransferSync', 'Start TransferSync check for domain: '.idn_to_ascii($params['sld']).'.'.idn_to_ascii($params['tld']), ' ', null, null);

    $EuroDNS = new EuroDNS([
        'Username' => $params['Username'],
        'Password' => $params['Password'],
        'TestMode' => $params['TestMode'],
    ]);

    $data = [
        'tld' => idn_to_ascii($params['tld']),
        'sld' => idn_to_ascii($params['sld']),
    ];

    $info = $EuroDNS->getInfo($data);
    if (!$info) {
        return;
    }
    if ($EuroDNS->getDomainStatus($EuroDNS->getResponse())) {
        $return['completed'] = true;
        $return['failed'] = false;
    } else {
        $return['completed'] = false;
        $return['failed'] = false;
    }

    if ($e = $EuroDNS->getError()) {
        $return['error'] = $e;
    }

    logModuleCall('EuroDNS', 'TransferSync', $EuroDNS->getRequest(), $EuroDNS->getResponse(), null, null);

    return $return;
}

function EuroDNS_Sync($params)
{
    $return = [];

    logModuleCall('EuroDNS', 'Sync', 'Start sync domain: '.idn_to_ascii($params['sld']).'.'.idn_to_ascii($params['tld']), ' ', null, null);

    $EuroDNS = new EuroDNS([
        'Username' => $params['Username'],
        'Password' => $params['Password'],
        'TestMode' => $params['TestMode'],
    ]);

    $data = [
        'tld' => idn_to_ascii($params['tld']),
        'sld' => idn_to_ascii($params['sld']),
    ];

    $info = $EuroDNS->getInfo($data);
    if (!$info) {
        if ('5111' === $EuroDNS->getErrorCode()) {
            logModuleCall('EuroDNS', 'Sync failed! Domain not exist', $return, $EuroDNS->getResponse(), null, null);

            return [
                'active' => false,
                'transferredAway' => true,
            ];
        }

        return;
    }

    if ($error = $EuroDNS->getError()) {
        $return['error'] = $error;
    }

    if ('5111' === $EuroDNS->getErrorCode()) {
        logModuleCall('EuroDNS', 'Sync failed! Domain not exist', $return, $EuroDNS->getResponse(), null, null);

        return [
            'active' => false,
            'transferredAway' => true,
        ];
    }

    //# Expiry Date
    $time = strtotime((string) $info->xpath('/response/resData/domain:expDate')[0]);
    $newformat = date('Y-m-d', $time);
    $return['expirydate'] = $newformat;
    $now = strtotime(date('Y-m-d', time()));

    if ($now > $time) {
        $command = 'UpdateClientDomain';
        $postData = [
            'domainid' => $params['domainid'],
            'status' => 'Expired',
        ];

        $results = localAPI($command, $postData);
    }

    logModuleCall('EuroDNS', 'Sync', $EuroDNS->getRequest(), $EuroDNS->getResponse(), null, null);

    return $return;
}

function EuroDNS_IDProtectToggle($params)
{
    $return = [];

    $EuroDNS = new EuroDNS([
        'Username' => $params['Username'],
        'Password' => $params['Password'],
        'TestMode' => $params['TestMode'],
    ]);

    $data = [
        'tld' => idn_to_ascii($params['tld']),
        'sld' => idn_to_ascii($params['sld']),
    ];

    $info = $EuroDNS->getInfo($data);
    if (!$info) {
        return;
    }
    if ($error = $EuroDNS->getError()) {
        $return['error'] = $error;
    } else {
        $res = $EuroDNS->toggleIDProtection($params);
        logModuleCall('EuroDNS', 'IDProtectionToggle', $EuroDNS->getRequest(), $EuroDNS->getResponse(), null, null);

        return $res;
    }

    return $return;
}

function EuroDNS_GetEPPCode($params)
{
    $EuroDNS = new EuroDNS([
        'Username' => $params['Username'],
        'Password' => $params['Password'],
        'TestMode' => $params['TestMode'],
    ]);

    $data = [
        'tld' => idn_to_ascii($params['tld']),
        'sld' => idn_to_ascii($params['sld']),
    ];

    $info = $EuroDNS->getInfo($data);
    if (!$info) {
        return;
    }

    if ($error = $EuroDNS->getError()) {
        return [
            'error' => $error,
        ];
    }

    $EPP = (string) $info->xpath('/response/resData/domain:authCode')[0];

    if ($EPP) {
        //$EPP = $EuroDNS->GetStringBetween($EPP, '[CDATA[', ']]');
        return [
            'eppcode' => $EPP,
        ];
    }

    $url = explode('?', $_SERVER['HTTP_REFERER'])[0];
    if (false !== strpos($url, 'clientarea.php')) {
        return [
            'error' => 'Click on <a href="'.$url.'?action=domaindetails&id='.$_GET['domainid'].'&modop=custom&a=TransferOut"> Transfer out </a>to generate a new EPP Code, then allow 10 - 15 minutes for it to process before clicking on "Get EPP Code"',
        ];
    }

    return [
        'error' => 'No EPP code available, Click on the "Transfer out" link to generate a new EPP Code, then allow 10 - 15 minutes for it to process before clicking on "Get EPP Code"',
    ];
}

function EuroDNS_ResendIRTPVerificationEmail($params)
{
    $EuroDNS = new EuroDNS([
        'Username' => $params['Username'],
        'Password' => $params['Password'],
        'TestMode' => $params['TestMode'],
    ]);

    $contacts = $EuroDNS->getContact($params);
    if (!$contacts) {
        return;
    }
    $data = [
        'resendValidation' => $contacts['Registrant']['Email'],
    ];

    if (!$EuroDNS->ResendValidationMail($data)) {
        if ($contacts['Registrant']['Email'] && !empty($contacts['Registrant']['Email'])) {
            logModuleCall('EuroDNS', 'Resend IRTP Verification Email', $EuroDNS->getRequest(), $EuroDNS->getError(), null, null);
        } else {
            logModuleCall('EuroDNS', 'Resend IRTP Verification Email', 'Could not generate Request check email address on registrant: '.$EuroDNS->getRequest(), $EuroDNS->getError(), null, null);
        }
    } else {
        logModuleCall('EuroDNS', 'Resend IRTP Verification Email', $EuroDNS->getRequest(), $EuroDNS->getResponse(), null, null);
    }
}

function EuroDNS_ClientAreaCustomButtonArray($params)
{
    $command = 'GetClientsDomains';
    $TransferOutInProgress = false;

    //$postData = array(
    //    'clientid' => $params["userid"],
    //    'stats' => true,
    //);
//
    //$results = localAPI($command, $postData);
//
    //$buttonarray = array();
//
    //foreach($results['domains']['domain'] as $domain)
    //{
    //    if($domain['domainname'] === ($params['sld'].'.'.$params['tld']) && $domain['status'] === 'Pending Transfer')
    //    {
    //        $buttonarray['Cancel Transfer out'] = 'CancelTransferOut';
    //        $TransferOutInProgress = true;
    //    }
    //
    //}
//
    if (!$TransferOutInProgress) {
        $buttonarray['Transfer out'] = 'TransferOut';
    }

    return $buttonarray;
}

function EuroDNS_ClientAreaAllowedFunctions($params)
{
    $command = 'GetClientsDomains';
    $TransferOutInProgress = false;

    //$postData = array(
    //    'clientid' => $params["userid"],
    //    'stats' => true,
    //);
//
    //$results = localAPI($command, $postData);
//
    //$buttonarray = array();
//
    //foreach($results['domains']['domain'] as $domain)
    //{
    //    if($domain['domainname'] === ($params['sld'].'.'.$params['tld']) && $domain['status'] === 'Pending Transfer')
    //    {
    //        $buttonarray['Cancel Transfer out'] = 'CancelTransferOut';
    //        $TransferOutInProgress = true;
    //    }
    //
    //}

    if (!$TransferOutInProgress) {
        $buttonarray['Transfer out'] = 'TransferOut';
    }

    return $buttonarray;
}

function EuroDNS_TransferOut($params)
{
    $EuroDNS = new EuroDNS([
        'Username' => $params['Username'],
        'Password' => $params['Password'],
        'TestMode' => $params['TestMode'],
    ]);

    $data = [
        'tld' => idn_to_ascii($params['tld']),
        'sld' => idn_to_ascii($params['sld']),
    ];

    $res = $EuroDNS->TransferOut($data);

    logModuleCall('EuroDNS', 'Transfer out request', $EuroDNS->getRequest(), $EuroDNS->getResponse(), null, null);

    if ($error = $EuroDNS->getError()) {
        return [
            'error' => $error,
        ];
    }

    return [
        'message' => 'Transfer out request was send successful',
    ];
}

function EuroDNS_ClientArea($params)
{
    if ($_GET['a']) {
        if ($_GET['b'] && $params['registrar'] && 'EuroDNS' === $params['registrar']) {
            $EuroDNS = new EuroDNS([]);
            $res = EuroDNS_TransferOut($params);

            if ($res['error']) {
                return $EuroDNS->GetErrorMessageFrontend('Transfer out', 'Transfer out request could not be send. Message: '.$res['error']);
            }

            return $EuroDNS->GetInfoMessageFrontend('Transfer out', 'Transfer out request was successful');
        }

        $url = explode('?', $_SERVER['HTTP_REFERER'])[0];
        $first = true;
        foreach ($_GET as $key => $val) {
            if (!$first) {
                $url .= '&'.$key.'='.$val;
            } else {
                $url .= '?'.$key.'='.$val;
            }

            $first = false;
        }

        $output = '
                <div class="eurodns-transferout-container" style="position: fixed; top: 5px; left: 5px; right: 5px; bottom: 5px; background: rgba(0,0,0,0.5); display: flex;">
                    <div class="eurodns-transferout-popup" style="height: 200px; width: 400px; background: white; border-radius: 5px; margin: auto auto;">
                        <div class="eurodns-transout-btn">
                        <div style="text-align: center; font-size: 18px; margin-top: 50px;"> Do you want to send Transfer out request?</div>
                        <div style="margin-top: 50px; text-align: center;">
                            <a class="btn" style="font-size: 20px; margin-right: 50px;" href="'.$url.'&b=go">Send</a>
                            <a class="btn" style="font-size: 20px;" href="javascript:void(0)" onclick="RemTransOutCont();">Cancel</a>
                        </div>
                        </div>
                    </div>
                </div>
                <script>
                    function RemTransOutCont()
                    {
                        jQuery(".eurodns-transferout-container").remove();
                    }
                </script>
            ';

        return $output;
    }
}

function EuroDNS_CancelTransferOut($params)
{
    $EuroDNS = new EuroDNS([
        'Username' => $params['Username'],
        'Password' => $params['Password'],
        'TestMode' => $params['TestMode'],
    ]);

    $data = [
        'tld' => idn_to_ascii($params['tld']),
        'sld' => idn_to_ascii($params['sld']),
    ];

    $res = $EuroDNS->CancelTransferOut($data);

    logModuleCall('EuroDNS', 'Cancel Transfer out request', $EuroDNS->getRequest(), $EuroDNS->getResponse(), null, null);

    if ($error = $EuroDNS->getError()) {
        return [
            'error' => $error,
        ];
    }

    return [
        'message' => 'Cancel Transfer out request was send successful',
    ];
}

function EuroDNS_AdminCustomButtonArray($params)
{
    $command = 'GetClientsDomains';
    $TransferOutInProgress = false;

    //$postData = array(
    //    'clientid' => $params["userid"],
    //    'stats' => true,
    //);
//
    //$results = localAPI($command, $postData);
//
    //$buttonarray = array();
//
    //foreach($results['domains']['domain'] as $domain)
    //{
    //    if($domain['domainname'] === ($params['sld'].'.'.$params['tld']) && $domain['status'] === 'Pending Transfer')
    //    {
    //        $buttonarray['Cancel Transfer out'] = 'CancelTransferOut';
    //        $TransferOutInProgress = true;
    //    }
    //
    //}

    if (!$TransferOutInProgress) {
        $buttonarray['Transfer out'] = 'TransferOut';
    }

    return $buttonarray;
}

//function EuroDNS_GetTldPricing($params)
//{
//    return;
//    $results = new ResultsList;
//
//    $extensionData = array();
//    $extension = array('tld' => 'ag','minPeriod' => 1, 'maxPeriod' => 10, 'registrationPrice' =>10, 'transferSecretRequired' => true);
//    $extensionData[] = $extension;
//    $extension = array('tld' => 'be','minPeriod' => 1, 'maxPeriod' => 10, 'registrationPrice' =>10, 'transferSecretRequired' => true);
//    $extensionData[] = $extension;
//    $extension = array('tld' => 'it','minPeriod' => 1, 'maxPeriod' => 10, 'registrationPrice' =>10, 'transferSecretRequired' => true);
//    $extensionData[] = $extension;
//
//    logModuleCall('EuroDNS', 'GetTldPricing', "register following domains: " , json_encode($extensionData) , null, null);
//
//    foreach ($extensionData as $extension)
//    {
//        // All the set methods can be chained and utilised together.
//        //$item = (new ImportItem)
//        //    ->setExtension($extension['tld'])
//        //    ->setMinYears($extension['minPeriod'])
//        //    ->setMaxYears($extension['maxPeriod'])
//        //    ->setRegisterPrice($extension['registrationPrice'])
//        //    ->setRenewPrice($extension['renewalPrice'])
//        //    ->setTransferPrice($extension['transferPrice'])
//        //    ->setRedemptionFeeDays($extension['redemptionDays'])
//        //    ->setRedemptionFeePrice($extension['redemptionFee'])
//        //    ->setCurrency($extension['currencyCode'])
//        //    ->setEppRequired($extension['transferSecretRequired']);
//
//        $item = (new ImportItem)
//        ->setExtension($extension['tld'])
//        ->setMinYears($extension['minPeriod'])
//        ->setMaxYears($extension['maxPeriod'])
//        ->setRegisterPrice($extension['registrationPrice'])
//        ->setEppRequired($extension['transferSecretRequired']);
//
//        $results[] = $item;
//    }
//
//    return $results;
//}
