<?php

// Please complete access to  MySQL in EuroDNS class.

if (false == function_exists('mysql_safequery')) {
    /**
     * FUNCTION mysql_safequery
     * Connects MySQL safely.
     *
     * @param string $query
     * @param array  $params
     *
     * @return mysql_query
     *                     works
     */
    function mysql_safequery($query, $params = false)
    {
        if ($params) {
            foreach ($params as &$v) {
                $v = mysql_real_escape_string($v);
            }

            $sql_query = vsprintf(str_replace('?', "'%s'", $query), $params);
            $sql_query = mysql_query($sql_query);
        } else {
            $sql_query = mysql_query($query);
        }

        return $sql_query;
    }
}

/**
 * Description of classEuroDns
 * This is a class for EuroDNS API its writed to WHMCS.
 */
class EuroDNS
{
    private $login = [];
    private $param = [];
    private $error;
    private $errorCode;
    private $request;
    private $response;
    private $pendingMessage;
    private $cacheFile = 'cached.ed';

    /**
     * FUNCTION __construct
     * Writes data to a log for API.
     */
    public function __construct(array $data)
    {
        $this->login = $data;
    }

    /**
     * FUNCTION getError
     * Get error.
     *
     * @param null
     *
     * @return string $this->error
     */
    public function getError()
    {
        return $this->error;
    }

    /**
     * FUNCTION getRequest
     * Get Request xml.
     *
     * @param null
     *
     * @return string $this->request
     */
    public function getRequest()
    {
        return $this->request;
    }

    /**
     * FUNCTION getResponse
     * Get Response xml.
     *
     * @param null
     *
     * @return string $this->response
     */
    public function getResponse()
    {
        return $this->response;
    }

    public function getErrorCode()
    {
        return $this->errorCode;
    }

    public function getPendingMessage()
    {
        return $this->pendingMessage;
    }

    /**
     * FUNCTION infoTld
     * Asks about the tld return a object containing the xml information about tld.
     *
     * @param string $tld
     *
     * @return $this->xmlToObject()
     */
    public function infoTld($tld)
    {
        $request = '<?xml version="1.0" encoding="UTF-8"?> 
                <request xmlns:tld="http://www.eurodns.com/tld"> 
                    <tld:info> 
                        <tld:name>'.$tld.'</tld:name> 
                    </tld:info> 
                </request>';
        $this->request = $request;

        if (!$response = $this->connect($request)) {
            // logModuleCall('EuroDNS','infoTld',  $response,$this->request,null,null);
            return false;
        }

        $this->response = $response;
        // logModuleCall('EuroDNS','infoTld',  $this->response,$this->request,null,null);
        $code = simplexml_load_string($response);

        if (!strstr($response, 'Not Found')) {
            if (1000 != $code->result->attributes()) {
                return 'false';
            }
        } else {
            return false;
        }

        return $this->xmlToObject($response, 'tld');
    }

    /**
     * FUNCTION Create
     * Registers domain.
     *
     * @param array $data
     * @param mixed $enablePrivacy
     *
     * @return string $this->returnResult()
     */
    public function Create($data, $enablePrivacy = false)
    {
        $this->param = $data;

        if (!$tldInfo = $this->infoTld($this->param['tld'])) {
            $result = mysql_safequery('SELECT * FROM eurodnstld WHERE tld=? LIMIT 1', [$this->param['tld']]);

            if ($row = mysql_fetch_array($result)) {
                $params = unserialize($row['params']);
                $trusteeBCAllowed = $params['trusteeBCAllowed'];
            }
        } else {
            $trusteeBCAllowed = (string) $tldInfo[0]->trusteeBCAllowed;
        }

        $request = '<?xml version="1.0" encoding="UTF-8"?> 
        <request  
                xmlns:domain="http://www.eurodns.com/domain"  
                xmlns:nameserver="http://www.eurodns.com/nameserver" 
                xmlns:contact="http://www.eurodns.com/contact" 
                xmlns:company="http://www.eurodns.com/company" 
                xmlns:extension="http://www.eurodns.com/extension"
                xmlns:secdns="http://www.eurodns.com/secdns"
                xmlns:nameserverprofile="http://www.eurodns.com/nameserverprofile"
                xmlns:zoneprofile="http://www.eurodns.com/zoneprofile"> 

               <domain:create> 
                  <domain:name>'.$this->param['sld'].'.'.$this->param['tld'].'</domain:name> 
                  <domain:year>'.$this->param['regperiod'].'</domain:year> 
                  <domain:renewal>'.$this->param['renewal'].'</domain:renewal> 
               </domain:create>
                '.$this->setNs('create').'
                '.$this->setContact('create', $trusteeBCAllowed).'
                '.$this->setAdditionalInformation('create', $enablePrivacy).'
        </request>';

        logModuleCall('EuroDNS', 'RegisterDomain', $request, $request, null, null);

        $this->request = $request;

        return $this->returnResult($request);
    }

    public function toggleIDProtection($data)
    {
        $this->param = $data;
        $request = '<?xml version="1.0" encoding="UTF-8"?> 
        <request 
                xmlns:domain="http://www.eurodns.com/domain"
                xmlns:nameserver="http://www.eurodns.com/nameserver"
                xmlns:nameserverprofile="http://www.eurodns.com/nameserverprofile"
                xmlns:zoneprofile="http://www.eurodns.com/zoneprofile">

            <domain:update> 
                <domain:name>'.$this->param['sld'].'.'.$this->param['tld'].'</domain:name> 
            </domain:update>
            <extension:create>
                <extension:service>
                    <service:domainprivacy>'.($data['protectenable'] ? 'Yes' : 'No').'</service:domainprivacy>
                </extension:service>
            </extension:create>
        </request>';

        return $this->returnResult($request);
    }

    /**
     * FUNCTION Transfer
     * Transfer domain.
     *
     * @param array $data
     *
     * @return string $this->returnResult()
     */
    public function Transfer($data)
    {
        $this->param = $data;

        if (!$tldInfo = $this->infoTld($this->param['tld'])) {
            return 'error';
        }

        $request = '<?xml version="1.0" encoding="UTF-8"?> 
        <request  
             xmlns:domain="http://www.eurodns.com/domain"  
             xmlns:nameserver="http://www.eurodns.com/nameserver"  
             xmlns:contact="http://www.eurodns.com/contact"> 
    
         <domain:transfer> 
           <domain:name>'.$this->param['sld'].'.'.$this->param['tld'].'</domain:name> 
           <domain:authcode>'.$this->param['transfersecret'].'</domain:authcode> 
         </domain:transfer>    
                  '.$this->setNs('create').'
                  '.$this->setContact('create', null).'
                  '.$this->setAdditionalInformation('create').'    
        </request>';

        $this->request = $request;

        return $this->returnResult($request);
    }

    /**
     * FUNCTION Transfer out to get EPP code
     * Transfer out domain.
     *
     * @param array $data
     *
     * @return string $this->returnResult()
     */
    public function TransferOut($data)
    {
        $this->param = $data;

        if (!$tldInfo = $this->infoTld($this->param['tld'])) {
            return 'error';
        }

        $request = '<?xml version="1.0" encoding="UTF-8"?> 
        <request xmlns:domain="http://www.eurodns.com/domain"> 
            <domain:transferout> 
                <domain:name>'.$this->param['sld'].'.'.$this->param['tld'].'</domain:name> 
            </domain:transferout>    
        </request>';

        $this->request = $request;

        return $this->returnResult($request);
    }

    /**
     * CancelTransferOut TODO: API NOT IMPLEMENTED YES.
     *
     * @param mixed $data
     */
    public function CancelTransferOut($data)
    {
        $this->param = $data;

        if (!$tldInfo = $this->infoTld($this->param['tld'])) {
            return false;
        }

        $request = '<?xml version="1.0" encoding="UTF-8"?> 
        <request xmlns:domain="http://www.eurodns.com/domain"> 
            <domain:transferout> 
                <domain:name>'.$this->param['sld'].'.'.$this->param['tld'].'</domain:name> 
            </domain:transferout>    
        </request>';

        $this->request = $request;

        return $this->returnResult($request);
    }

    /**
     * FUNCTION Renew
     * Renew domain.
     *
     * @param array $data
     *
     * @return string $this->returnResult()
     */
    public function Renew($data)
    {
        $this->param = $data;
        $request = '<?xml version="1.0" encoding="UTF-8"?> 
        <request xmlns:domain="http://www.eurodns.com/domain"> 
           <domain:renew> 
               <domain:name>'.$this->param['sld'].'.'.$this->param['tld'].'</domain:name> 
               <domain:year>'.$this->param['regperiod'].'</domain:year> 
           </domain:renew>         
        </request>';

        $this->request = $request;

        // return ($this->returnResult($request));
        return $this->connect($request);
    }

    /**
     * FUNCTION Reactivate domain
     * Reactivate domain.
     *
     * @param array $data
     *
     * @return string $this->returnResult()
     */
    public function Reactivate($data)
    {
        $this->param = $data;
        $request = '<?xml version="1.0" encoding="UTF-8"?> 
        <request xmlns:domain="http://www.eurodns.com/domain"> 
           <domain:reactivate> 
               <domain:name>'.$this->param['sld'].'.'.$this->param['tld'].'</domain:name> 
               <domain:year>'.$this->param['regperiod'].'</domain:year>
           </domain:reactivate>         
        </request>';

        $this->request = $request;

        return $this->connect($request);
        // return ($this->returnResult($request));
    }

    /**
     * FUNCTION getInfo
     * Retrieves data domain.
     * Owner $return = contact  return contact ; $return = ns return name server.
     *
     * @param array  $data
     * @param string $return
     *
     * @return array $ns or array $this->xmlToObject()
     */
    public function getInfo($data, $return = '')
    {
        $this->param = $data;
        $request = '<?xml version="1.0" encoding="UTF-8"?> 
        <request xmlns:domain="http://www.eurodns.com/domain"> 
           <domain:info> 
             <domain:name>'.$this->param['sld'].'.'.$this->param['tld'].'</domain:name> 
           </domain:info> 
        </request>';
        $this->request = $request;

        $response = $this->connect($request);

        if (!$response) {
            return false;
        }

        $this->response = $response;
        $code = simplexml_load_string($response);

        if (strstr($response, 'Not Found') || empty($response)) {
            return 'ERROR: no response';
        }

        if (((string) $code->result->attributes()[0]) === '5111') {
            return $code;
        }

        $pendingtmp = $code->xpath('/response/resData/domain:pending');

        if (!empty($pendingtmp)) {
            if (!empty($code->xpath("/response/resData/domain:pending[@status='yes']"))) {
                $this->pendingMessage = (string) $code->xpath('/response/resData/domain:pending/domain:pendingAction')[0];
            }
        }

        // if($code->xpath("/response/resData/domain:pending[@status='Large']"))

        // Gets the name  server of the string XML  and returns ns
        if ('ns' == $return) {
            $ns = [];
            $xml = new SimpleXMLElement($this->validateResponse($response));

            foreach ($xml->extension as $value) {
                foreach ($value->children('http://www.eurodns.com/eurodns') as $value2) {
                    foreach ($value2->children('http://www.eurodns.com/domain') as $value3) {
                        $ns[] = (string) $value3[0];
                    }
                }
            }

            return $ns;
        }

        return $code;
    }

    /**
     * FUNCTION zoneInfo
     * Gets the list of records for a zone.
     *
     * @param string $data
     *
     * @return string $this->returnResult()
     */
    public function zoneInfo($data)
    {
        $this->param = $data;

        $this->request = '<?xml version="1.0" encoding="UTF-8"?>
                    <request xmlns:zone="http://www.eurodns.com/zone">
                        <zone:info>
                            <zone:name>'.$this->param['sld'].'.'.$this->param['tld'].'</zone:name>
                        </zone:info>
                    </request>';

        if (!$this->connect($this->request)) {
            return false;
        }

        return $this->response;
    }

    /**
     * FUNCTION updateNs
     * Update name server.
     *
     * @param array $data
     *
     * @return string $this->returnResult()
     */
    public function updateDNS($data)
    {
        $this->param = $data;
        $request = '<?xml version="1.0" encoding="UTF-8"?> 
        <request xmlns:zone="http://www.eurodns.com/zone">
            <zone:update> 
                <zone:name>'.$this->param['sld'].'.'.$this->param['tld'].'</zone:name> 
                    <zone:records>
                        '.$this->setDNS($data['dns']).'
                    </zone:records>
            </zone:update>
        </request>';

        return $this->returnResult($request);
    }

    /**
     * add records or remove
     * return: xml list of entry's.
     *
     * @param mixed $data
     */
    public function setDNS($data)
    {
        $out = '';
        foreach ($data as $method => $record) {
            $out .= '<zone:'.$method.'>';

            foreach ($record as $value) {
                $id = (('' != $value['recid']) ? ' id="'.$value['recid'].'"' : '');
                unset($value['recid']);

                if ('remove' == $method) {
                    $out .= '<zone:record '.$id.'/>';
                } else {
                    $out .= (('remove' == $method) ? '<zone:record '.$id.'/>' : '<zone:record'.$id.'>');

                    foreach ($value as $name => $v) {
                        $out .= '<record:'.$name.'>'.$v.'</record:'.$name.'>';
                    }

                    $out .= '</zone:record>';
                }
            }

            $out .= '</zone:'.$method.'>';
        }

        return $out;
    }

    /**
     * FUNCTION updateNs
     * Update name server.
     *
     * @param array $data
     *
     * @return string $this->returnResult()
     */
    public function updateNs($data)
    {
        // data to the query
        $this->param = $data;
        $request = '<?xml version="1.0" encoding="UTF-8"?> 
        <request xmlns:domain="http://www.eurodns.com/domain" xmlns:nameserver="http://www.eurodns.com/nameserver">
            <domain:update> 
                <domain:name>'.$this->param['sld'].'.'.$this->param['tld'].'</domain:name> 
            </domain:update>
            '.$this->setNs('update').'
            '.$this->setAdditionalInformation('update', false, true).'
        </request>';

        return $this->returnResult($request);
    }

    /**
     * FUNCTION getContact
     * Returns the domain contact.
     *
     * @param array $data
     *
     * @return array $this->xmlToObject()
     */
    public function getContact($data)
    {
        $xml = $this->getInfo($data, 'contact');
        $values = [];

        foreach ($xml->xpath('/response/resData/domain:contact') as $value) {
            $request = '<?xml version="1.0" encoding="UTF-8"?> 
            <request xmlns:contact="http://www.eurodns.com/contact">
             <contact:info> 
               <contact:ID>'.$value.'</contact:ID> 
             </contact:info>
             </request>
            ';

            $this->request = $request;
            $response = $this->connect($request);

            if (!$response) {
                return false;
            }

            $this->response = $response;
            $code = simplexml_load_string($response);

            if (1000 != $code->result->attributes()) {
                return $values['error'] = 'error';
            }

            $xml2[] = $this->xmlToObject($response, 'contact');
        }

        $user = explode(' ', $xml2[0][0]->postalInfo->name);
        $values['Registrant']['First Name'] = $user[1];
        $values['Registrant']['Last Name'] = $user[0];
        $values['Registrant']['address1'] = (string) $xml2[0][0]->postalInfo->addr->street;
        $values['Registrant']['company'] = (string) $xml2[0][0]->postalInfo->org;
        $values['Registrant']['city'] = (string) $xml2[0][0]->postalInfo->addr->city;
        $values['Registrant']['postcode'] = (string) $xml2[0][0]->postalInfo->addr->pc;
        $values['Registrant']['country'] = (string) $xml2[0][0]->postalInfo->addr->cc; // code country
        $values['Registrant']['phonenumber'] = (string) $xml2[0][0]->postalInfo->voice;
        $values['Registrant']['Fax'] = (string) $xml2[0][0]->postalInfo->fax;
        $values['Registrant']['Email'] = (string) $xml2[0][0]->postalInfo->email;

        // Tech datails
        $user = explode(' ', $xml2[2][0]->postalInfo->name);
        $values['Tech']['First Name'] = (string) $user[1];
        $values['Tech']['Last Name'] = $user[0];
        $values['Tech']['address1'] = (string) $xml2[2][0]->postalInfo->addr->street;
        $values['Tech']['company'] = (string) $xml2[2][0]->postalInfo->org;
        $values['Tech']['city'] = (string) $xml2[2][0]->postalInfo->addr->city;
        $values['Tech']['postcode'] = (string) $xml2[2][0]->postalInfo->addr->pc;
        $values['Tech']['country'] = (string) $xml2[2][0]->postalInfo->addr->cc; // code country
        $values['Tech']['phonenumber'] = (string) $xml2[2][0]->postalInfo->voice;
        $values['Tech']['Fax'] = (string) $xml2[2][0]->postalInfo->fax;
        $values['Tech']['Email'] = (string) $xml2[2][0]->postalInfo->email;

        // Biling dat
        $user = explode(' ', $xml2[1][0]->postalInfo->name);
        $values['Billing']['First Name'] = $user[1];
        $values['Billing']['Last Name'] = $user[0];
        $values['Billing']['address1'] = (string) $xml2[1][0]->postalInfo->addr->street;
        $values['Billing']['company'] = (string) $xml2[1][0]->postalInfo->org;
        $values['Billing']['city'] = (string) $xml2[1][0]->postalInfo->addr->city;
        $values['Billing']['postcode'] = (string) $xml2[1][0]->postalInfo->addr->pc;
        $values['Billing']['country'] = (string) $xml2[1][0]->postalInfo->addr->cc; // code country
        $values['Billing']['phonenumber'] = (string) $xml2[1][0]->postalInfo->voice;
        $values['Billing']['Fax'] = (string) $xml2[1][0]->postalInfo->fax;
        $values['Billing']['Email'] = (string) $xml2[1][0]->postalInfo->email;

        // Admin datails
        $user = explode(' ', $xml2[3][0]->postalInfo->name);
        $values['Admin']['First Name'] = (string) $user[1];
        $values['Admin']['Last Name'] = $user[0];
        $values['Admin']['address1'] = (string) $xml2[3][0]->postalInfo->addr->street;
        $values['Admin']['company'] = (string) $xml2[3][0]->postalInfo->org;
        $values['Admin']['city'] = (string) $xml2[3][0]->postalInfo->addr->city;
        $values['Admin']['postcode'] = (string) $xml2[3][0]->postalInfo->addr->pc;
        $values['Admin']['country'] = (string) $xml2[3][0]->postalInfo->addr->cc; // code country
        $values['Admin']['phonenumber'] = (string) $xml2[3][0]->postalInfo->voice;
        $values['Admin']['Fax'] = (string) $xml2[3][0]->postalInfo->fax;
        $values['Admin']['Email'] = (string) $xml2[3][0]->postalInfo->email;

        return $values;
    }

    /**
     * FUNCTION updateContact
     * Update Contact details.
     *
     * @param array $data
     *
     * @return string $this->returnResult()
     */
    public function updateContact($data)
    {
        $this->param = $data;

        $request = '<?xml version="1.0" encoding="UTF-8"?> 
        <request xmlns:domain="http://www.eurodns.com/domain" xmlns:contact="http://www.eurodns.com/contact">
          <domain:update> 
            <domain:name>'.$this->param['sld'].'.'.$this->param['tld'].'</domain:name> 
          </domain:update>       
             '.$this->setContact('update', null).'
             '.$this->setAdditionalInformation('update', false, true).'
        </request>';

        $this->request = $request;
        // return ($this->returnResult($request));
        return $this->connect($request);
    }

    /**
     * FUNCTION Lock
     * Lock domain.
     *
     * @param array $data
     *
     * @return string $this->returnResult()
     */
    public function Lock($data)
    {
        $this->param = $data;
        $request = '<?xml version="1.0" encoding="UTF-8"?> 
        <request xmlns:domain="http://www.eurodns.com/domain">     
            <domain:lock> 
                <domain:name>'.$this->param['sld'].'.'.$this->param['tld'].'</domain:name> 
            </domain:lock>     
        </request>';
        $this->request = $request;
        // zwraca success lub error
        // return ($this->returnResult($request));
        return $this->connect($request);
    }

    /**
     * FUNCTION UnLock
     * Unlock domain.
     *
     * @param array $data
     *
     * @return string $this->returnResult()
     */
    public function UnLock($data)
    {
        $this->param = $data;
        $request = '<?xml version="1.0" encoding="UTF-8"?> 
        <request xmlns:domain="http://www.eurodns.com/domain">     
            <domain:unlock> 
                <domain:name>'.$this->param['sld'].'.'.$this->param['tld'].'</domain:name> 
            </domain:unlock>     
        </request>';
        $this->request = $request;
        // zwraca success lub error
        // return ($this->returnResult($request));
        return $this->connect($request);
    }

    /**
     * FUNCTION SetRenewalMode
     * Domain renewal mode autoexpire or autorenew.
     *
     * @param array $data
     *
     * @return string $this->returnResult()
     */
    public function SetRenewalMode($data)
    {
        $this->param = $data;
        if ('autoexpire' != $this->param['renewal'] && 'autorenew' != $this->param['renewal']) {
            return 'erorr';
        }

        $request = '<?xml version="1.0" encoding="UTF-8"?> 
        <request xmlns:ip="http://www.eurodns.com/"> 
            <domain:setrenewalmode> 
                <domain:name>'.$this->param['sld'].'.'.$this->param['tld'].'</domain:name> 
                <domain:renewal>'.$this->param['renewal'].'</domain:renewal> 
            </domain:setrenewalmode> 
        </request>';
        $this->request = $request;
        // return success or error
        // return ($this->returnResult($request));
        return $this->connect($request);
    }

    public function ResendValidationMail($data)
    {
        $this->param = $data;

        if (!$this->param['resendValidation']) {
            return false;
        }

        $request = '<?xml version="1.0" encoding="UTF-8"?> 
        <request xmlns:EuroDNS="http://www.eurodns.com/" xmlns:domain="http://www.eurodns.com/"> 
            <email:resendValidation> 
                <email:address>'.$this->param['resendValidation'].'</email:address> 
            </email:resendValidation> 
        </request>';
        $this->request = $request;
        // return success or error
        // return ($this->returnResult($request));
        return $this->connect($request);
    }

    /**
     * FUNCTION check
     * Checks the availability of a domain.
     *
     * @param array $data
     *
     * @return array $values
     */
    public function check($data)
    {
        $this->param = $data;
        $values = [];

        $request = '<?xml version="1.0" encoding="UTF-8"?> 
        <request xmlns:domain="http://www.eurodns.com/domain"> 
            <domain:check> 
                <domain:name>'.$this->param['sld'].'.'.$this->param['tld'].'</domain:name> 
            </domain:check> 
        </request>';
        $this->request = $request;
        $response = $this->connect($request);

        if (!$response) {
            return false;
        }

        $this->response = $response;

        // checks the response code
        if (!$this->getCode($response)) {
            return false;
        }

        return true;
    }

    public function getDomainStatus($response)
    {
        try {
            $xmlDoc = new DOMDocument();
            $xmlDoc->loadXML($this->response);

            $xpath = new \DOMXPath($xmlDoc);
            $status = $xpath->query('//domain:status');

            if (1000 == $status[0]->getAttribute('code')) {
                return true;
            }

            return false;
        } catch (Exception $e) {
            return false;
        }
    }

    public function GetDNSForm($records, $actionUrl, $token, $DID)
    {
        $xmlDoc = new DOMDocument();
        $response = preg_replace('/(\>(?!\<).*?)((\&)\3*)(.*?\<)/', '$1&amp;$4', $records);
        $response = utf8_encode($response);
        $xmlDoc->loadXML($response);
        $records = $xmlDoc->getElementsByTagName('record');

        $recordsArray = [];

        foreach ($records as $record) {
            $ratmp = [];
            $ratmp['id'] = $record->getAttribute('id');
            foreach ($record->childNodes as $rec) {
                if (!empty($rec->localName)) {
                    $ratmp[$rec->localName] = $rec->textContent;
                }
            }
            $recordsArray[] = $ratmp;
        }

        $htmlOutput = '<div id="form-container"><div id="dns_info_message" ><strong>INFO!</strong><br>
		<div>* If you change TTL of MX or/and TXT, be sure to change all TTL of MX or/and TXT records to the same.</div>
		<div>* If you want to change NS records, please select Nameservers in the menu.</div>
		</div><form id="form-submit-dns" class="form-horizontal" method="post" action="'.$actionUrl.'" _lpchecked="1">';
        $htmlOutput .= '<input type="hidden" name="token" value="'.$token.'"></input>';
        $htmlOutput .= '<input type="hidden" name="sub" value="save"></input>';
        $htmlOutput .= '<input type="hidden" name="domainid" value="'.$DID.'"></input>';
        $htmlOutput .= '<div id="dns_record_table">';

        foreach ($recordsArray as $re) {
            if ('SOA' !== $re['type'] && 'NS' !== $re['type']) {
                $htmlOutput .= $this->GetListOfRecords($re, false);
            }
        }

        $htmlOutput .= $this->GetListOfRecords(['host' => '', 'type' => 'A', 'data' => '', 'ttl' => '3600'], true);
        $htmlOutput .= '</div>';
        $htmlOutput .= '<p class="text-center">
						<input type="submit" value="Save Changes" class="btn btn-primary" /> 
						<input type="reset" value="Cancel Changes" class="btn btn-default" />
					</p>';
        $htmlOutput .= '</form></div>';

        return $this->GetJSReplaceCode($htmlOutput, $actionUrl);
    }

    public function startsWith($haystack, $needle)
    {
        $length = strlen($needle);

        return substr($haystack, 0, $length) === $needle;
    }

    public function validateResponse($str)
    {
        if (1 == preg_match('/<domain:authCode>(.*?)<\/domain:authCode>/', $str, $match)) {
            $authCode = htmlspecialchars($match[1], ENT_QUOTES, 'UTF-8', false);

            return preg_replace('/<domain:authCode>(.+)<\/domain:authCode>/', '<domain:authCode>'.$authCode.'</domain:authCode>', $str);
        }

        return $str;
    }

    public function GetErrorMessage($title, $message)
    {
        return '<div class="errorbox"><span class="title">'.$title.'</span> <br> '.$message.' </div>';
    }

    public function GetInfoMessage($title, $message)
    {
        return '<div class="infobox"><span class="title">'.$title.'</span> <br> '.$message.' </div>';
    }

    public function GetInfoMessageFrontend($title, $message)
    {
        return '<div class="alert alert-info" style="text-align: center;"><span class="title" style="font-weight: bold;">'.$title.'</span> <br> '.$message.' </div>';
    }

    public function GetErrorMessageFrontend($title, $message)
    {
        return '<div class="alert alert-danger" style="text-align: center;"><span class="title" style="font-weight: bold;">'.$title.'</span> <br> '.$message.' </div>';
    }

    public function UpdateContactDetails(&$values)
    {
        if (array_key_exists('Address 1', $values)) {
            $values['address1'] = $values['Address 1'];
        }

        if (array_key_exists('City', $values)) {
            $values['city'] = $values['City'];
        }

        if (array_key_exists('Postcode', $values)) {
            $values['postcode'] = $values['Postcode'];
        }

        if (array_key_exists('Country', $values)) {
            $values['country'] = $values['Country'];
        }

        if (array_key_exists('Phone Number', $values)) {
            $values['phonenumber'] = $values['Phone Number'];
        }
    }

    public function GetStringBetween($string, $start, $end)
    {
        $string = ' '.$string;
        $ini = strpos($string, $start);
        if (0 == $ini) {
            return '';
        }
        $ini += strlen($start);
        $len = strpos($string, $end, $ini) - $ini;

        return substr($string, $ini, $len);
    }

    public function getCachedInfo($pathArray, $requestIfNotCached = false)
    {
        $tldCached = $this->getCachedInfoHelper($pathArray);

        if (!$tldCached && $requestIfNotCached && count($pathArray) > 1) {
            $tldInfo = $this->infoTld($pathArray[1]);

            if (empty($pathArray[1])) {
                $this->removeValueFromCache($this->getCacheFile(), $pathArray);
            }

            if ($tldInfo) {
                $array = [];

                foreach ($tldInfo[0] as $k => $v) {
                    $array[$k] = (string) $v;
                }

                $this->addInfoToCache(['tld', $pathArray[1]], $array);
                // $tldCached = $this->getValueHelper(array('tld'=>array($pathArray[1]=>$array)),$pathArray);
                $tldCached = $tldInfo[$pathArray[2]];
            }
        }

        return $tldCached;
    }

    public function addInfoToCache($pathArray, $value)
    {
        if (!is_file(__DIR__.'/'.$this->cacheFile)) {
            $arr = [];
            if (!file_put_contents(__DIR__.'/'.$this->cacheFile, json_encode($arr))) {
                return false;
            }

            chmod(__DIR__.'/'.$this->cacheFile, 0775);
        }

        $content = file_get_contents(__DIR__.'/'.$this->cacheFile);
        $arr = json_decode($content, true);
        $result = &$arr;

        foreach ($pathArray as $key) {
            if (array_key_exists($key, $result)) {
                $result = &$result[$key];
            } else {
                $result[$key] = [];
                $result = &$result[$key];
            }
        }

        $result = $value;

        if (!file_put_contents(__DIR__.'/'.$this->cacheFile, json_encode($arr))) {
            return false;
        }

        return true;
    }

    public function getCachedInfoHelper($pathArray)
    {
        $arr = $this->getCacheFile();
        if ($arr) {
            return $this->getValueHelper($arr, $pathArray);
        }

        return null;
    }

    public function getCacheFile()
    {
        if (is_file(__DIR__.'/'.$this->cacheFile)) {
            $content = file_get_contents(__DIR__.'/'.$this->cacheFile);

            if (!$content) {
                return false;
            }

            return json_decode($content, true);
        }

        return false;
    }

    public function getValueHelper($arr, $pathArray)
    {
        $result = &$arr;

        foreach ($pathArray as $key) {
            if (array_key_exists($key, $result)) {
                $result = &$result[$key];
            } else {
                return false;
            }
        }

        return $result;
    }

    public function removeValueFromCache($arr, $pathArray)
    {
        $result = &$arr;

        foreach ($pathArray as $key) {
            if (array_key_exists($key, $result)) {
                unset($result[$key]);
            }
        }

        if (!file_put_contents(__DIR__.'/'.$this->cacheFile, json_encode($arr))) {
            return false;
        }

        return true;
    }

    /**
     * FUNCTION connect
     * Connect to the api and returns xml response.
     *
     * @param string $request
     *
     * @return string $response
     */
    private function connect($request)
    {
        if ('on' == $this->login['TestMode']) {
            define('URL', 'https://secure.tryout-eurodns.com:20015/v2/');
            define('USERNAME', str_replace('_prod_', '_test_', $this->login['Username']));
            define('PASSWORD', 'MD5'.md5($this->login['Password']));
        } else {
            define('URL', 'https://secure.api-eurodns.com:20015/v2/');
            define('USERNAME', $this->login['Username']);
            define('PASSWORD', 'MD5'.md5($this->login['Password']));
        }

        if (empty(USERNAME) || empty(PASSWORD)) {
            logModuleCall('EuroDNS', 'connect', 'No username and/or Password set, please add your account under Setup -> Product/Services -> Domain Registrars -> EuroDNS', 'connection request', null, null);

            return false;
        }

        $cUrl = curl_init();
        curl_setopt($cUrl, CURLOPT_USERPWD, USERNAME.':'.PASSWORD);
        curl_setopt($cUrl, CURLOPT_URL, URL);
        curl_setopt($cUrl, CURLOPT_SSL_VERIFYPEER, 1);
        curl_setopt($cUrl, CURLOPT_SSL_VERIFYHOST, 1);
        curl_setopt($cUrl, CURLOPT_VERBOSE, 0);
        curl_setopt($cUrl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($cUrl, CURLOPT_POST, 1);
        curl_setopt($cUrl, CURLOPT_POSTFIELDS, 'xml='.urlencode(urlencode($request)));

        $response = curl_exec($cUrl);
        $this->errorCode = $this->getCode($response);
        $response = preg_replace('/(\>(?!\<).*?)((\&)\3*)(.*?\<)/', '$1&amp;$4', $response);
        $response = utf8_encode($response);
        $this->response = $response;

        if (!empty($this->error)) {
            // $ex = new Exception();
            // $trace = $ex->getTrace();
            // logModuleCall('EuroDNS','connect', 'Not able to connect to Server or something else: <br> '.json_encode($trace).'<br>'.$request,$this->error.'<br>'.$res,null,null);
            logModuleCall('EuroDNS', 'connect', 'Not able to connect to Server or something else: <br> '.$request, $this->error.'<br>'.$this->errorCode, null, null);

            return false;
        }

        if (0 != curl_errno($cUrl)) {
            return false;
        }

        curl_close($cUrl);

        return $response;
    }

    /**
     * FUNCTION xmlToObject
     * Change the xml string to  array contains xml object.
     *
     * @param string $xml
     * @param string $type
     *
     * @return array $array
     */
    private function xmlToObject($xml, $type)
    {
        $array = [];
        $xml = new SimpleXMLElement($this->validateResponse($xml));

        foreach ($xml->resData as $value) {
            $array[] = $value->children('http://www.eurodns.com/'.$type);
        }

        return $array;
    }

    /**
     * FUNCTION setAdditionalInformation
     * Set custom fields for tld.
     *
     * @param string $action
     * @param mixed  $security
     * @param mixed  $nsUpdate
     *
     * @return string $extension
     */
    private function setAdditionalInformation($action, $security = false, $nsUpdate = false)
    {
        $additional = null;
        $extension = null;
        $tld = strtolower($this->param['tld']);
        $type = $this->param['additionalfields']['type'];

        if ('com' == $tld || 'net' == $tld) {
            // foreach ($this->param['additionalfields'] as $k => $v) {
            //    $additional .= "<extension:{$k}>{$v}</extension:{$k}>\n";
            // }

            $extension = '<extension:'.$action.'>
                <extension:service>
                    <service:domainprivacy>'.($security ? 'Yes' : 'No').'</service:domainprivacy>
                </extension:service>
                <extension:data tld="com">
                    <extension:registry>
                        <registry:language>ENG</registry:language>
                    </extension:registry>
                </extension:data>
            </extension:'.$action.'>';

            $nsUpdate = true;

        // if ($additional && !empty($additional)) {
            //    $extension .= "<extension:{$action}>\n";
            //    $extension .= "     {$additional}";
//
            //    $extension .= '<extension:data tld="com">
            //                    <extension:registry>
            //                        <registry:language>ENG</registry:language>
            //                    </extension:registry>
            //                </extension:data>';
//
            //    $extension .= "</extension:{$action}>\n";
            // }
        } elseif ('tel' == $tld) {
            foreach ($this->param['additionalfields'] as $k => $v) {
                $additional .= "<entity:{$k}>{$v}</entity:{$k}>\n";
            }
            $extension = '<extension:'.$action.'> 
                 <extension:data tld="tel"> 
                    <extension:registration> 
                       <registration:entity> 
                           '.$additional.'
                       </registration:entity> 
                    </extension:registration> 
                 </extension:data> 
               </extension:'.$action.'>';

        // for COMPANY
        } elseif ('Company' == $type && ('fr' == $tld || 'asso.fr' == $tld || 'com.fr' == $tld || 'tm.fr' == $tld || 'pm' == $tld || 're' == $tld || 'wf' == $tld || 'yt' == $tld)) {
            foreach ($this->param['additionalfields'] as $k => $v) {
                if ('type' != $k && 'siren' == $k) {
                    $additional .= "<extension:{$k}>{$v}</extension:{$k}>\n";
                }
            }
            $extension = '<extension:'.$action.'> 
                                 '.$additional.' 
              </extension:'.$action.'>';
        // Private Owner
        } elseif ('Private Owner' == $type && ('fr' == $tld || 'asso.fr' == $tld || 'com.fr' == $tld || 'tm.fr' == $tld || 'pm' == $tld || 're' == $tld || 'wf' == $tld || 'yt' == $tld)) {
            foreach ($this->param['additionalfields'] as $k => $v) {
                if ('type' != $k && 'siren' != $k) {
                    $additional .= "<extension:{$k}>{$v}</extension:{$k}>\n";
                }
            }
            $extension = '<extension:'.$action.'> 
                                 '.$additional.' 
              </extension:'.$action.'>';
        } elseif ('nl' == $tld) {
            $extension = '<extension:'.$action.'> 
                <extension:data tld="nl"> 
					<extension:domain>   
						<domain:legal_form>
							ANDERS 
						</domain:legal_form>
						</extension:domain>
					</extension:data> 
             </extension:'.$action.'>';
        } elseif ('com.pt' == $tld || 'pt' == $tld || 'org.pt' == $tld) {
            $extension = '<extension:'.$action.'> 
                <extension:data tld="'.$tld.'"> 
					<extension:registry>   
						<registry:identification>'.$this->param['additionalfields']['identification'].'</registry:identification>
						</extension:registry>
					</extension:data> 
             </extension:'.$action.'>';
        } elseif ('it' == $tld) {
            // if ('company' == $this->param['additionalfields']['registrationType']) {
            //    $additional = '<extension:company>
            //             <company:name>'.$this->param['additionalfields']['name'].'</company:name>
            //             <company:vatNumber>'.$this->param['additionalfields']['vatNumber'].'</company:vatNumber>
            //             <company:registrationNumber>'.$this->param['additionalfields']['registrationNumber'].'</company:registrationNumber>
            //             <company:registrationDate>'.$this->param['additionalfields']['registrationDate'].'</company:registrationDate>
            //         </extension:company>';
            // }
            // if ('eurodns' != $this->param['additionalfields']['registrationType']) {
            //    $additional .= '
            //        <extension:contact>
            //             <contact:type>org</contact:type>
            //             <contact:birthDate>'.$this->param['additionalfields']['birthDate'].'</contact:birthDate>
            //             <contact:birthCity>'.$this->param['additionalfields']['birthCity'].'</contact:birthCity>
            //             <contact:birthCityZipCode>'.$this->param['additionalfields']['birthCityZipCode'].'</contact:birthCityZipCode>
            //             <contact:birthCountryCode>'.$this->param['additionalfields']['birthCountryCode'].'</contact:birthCountryCode>
            //             <contact:passportNumber>'.$this->param['additionalfields']['passportNumber'].'</contact:passportNumber>
            //         </extension:contact>';
            // }
            // $extension = '<extension:'.$action.'>
            //    <extension:data tld="it">
            //        <extension:domain>
            //            <domain:registrationType>'.$this->param['additionalfields']['registrationType'].'</domain:registrationType>
            //         </extension:domain>
            //        '.$additional.'
            //    </extension:data>
            // </extension:'.$action.'>';

            $entityType = ['Italian and foreign natural persons' => '1', 'Companies/one man companies' => '2',
                'Freelance workers/professionals' => '3', 'non-profit organizations' => '4', 'public organizations' => '5', 'other subjects' => '6', 'non natural foreigners' => '7', ];

            $extension = '<extension:'.$action.'> 
                <extension:data tld="it"> 
                    <extension:registry> 
                        <registry:a_itprovince>'.$this->param['state'].'</registry:a_itprovince>
                        <registry:entitytype>'.$entityType[$this->param['additionalfields']['Legal Type']].'</registry:entitytype>
                        <registry:nationality>'.$this->param['country_code'].'</registry:nationality>
                        <registry:o_itprovince>'.$this->param['state'].'</registry:o_itprovince>
                        <registry:regcode>'.$this->param['additionalfields']['Tax ID'].'</registry:regcode>
                        <registry:t_itprovince>'.$this->param['state'].'</registry:t_itprovince>
                        <registry:usetrusteecontacts>0</registry:usetrusteecontacts>
                     </extension:registry> 
                </extension:data> 
             </extension:'.$action.'>';

            $nsUpdate = true;
        } elseif ('se' == $tld) {
            foreach ($this->param['additionalfields'] as $k => $v) {
                $additional .= "<extension:{$k}>{$v}</extension:{$k}>\n";
            }
            $extension = '<extension:'.$action.'> 
                '.$additional.'
             </extension:'.$action.'>';
        } elseif ('vn' == $tld) {
            foreach ($this->param['additionalfields'] as $k => $v) {
                $additional .= "<extension:{$k}>{$v}</extension:{$k}>\n";
            }
            $extension = '<extension:'.$action.'> 
                '.$additional.'
             </extension:'.$action.'>';
        } elseif ('xxx' == $tld) {
            foreach ($this->param['additionalfields'] as $k => $v) {
                if ('period' != $k) {
                    $additional .= "\n                        <entity:{$k}>{$v}</entity:{$k}>";
                }
            }
            $extension = '<extension:'.$action.'> 
                <extension:data tld="xxx"> 
                   <extension:registration> 
                      <registration:period>'.$this->param['additionalfields']['period'].'</registration:period> 
                   </extension:registration> 
                      <registration:entity>'.$additional.'         
                      </registration:entity> 
                </extension:data> 
             </extension:'.$action.'>';
        } elseif ('ca' == $tld) {
            $extension = '<extension:'.$action.'> 
                <extension:data tld="ca"> 
                    <extension:registry> 
                        <registry:a_province>'.$this->param['state'].'</registry:a_province>
                        <registry:o_province>'.$this->param['state'].'</registry:o_province>
                        <registry:t_province>'.$this->param['state'].'</registry:t_province>
                     </extension:registry> 
                    <extension:domain>
                        <domain:a_caprovince>'.$this->param['state'].'</domain:a_caprovince>
                        <domain:b_caprovince>'.$this->param['state'].'</domain:b_caprovince>
                        <domain:caapproveagreement>'.('on' === $this->param['additionalfields']['CIRA Agreement'] ? '1' : '0').'</domain:caapproveagreement>
                        <domain:caregistrantlegaltype>'.$this->getRegistrantLegalType($this->param['additionalfields']['Legal Type']).'</domain:caregistrantlegaltype>
                        <domain:o_caprovince>'.$this->param['state'].'</domain:o_caprovince>
                        <domain:t_caprovince>'.$this->param['state'].'</domain:t_caprovince>
                        <domain:trademarkid></domain:trademarkid>
                    </extension:domain>
                </extension:data> 
             </extension:'.$action.'>';

            $nsUpdate = true;
        } elseif ('training' == $tld) {
            $extension = '<extension:'.$action.'>
                <extension:service>
                    <service:domainprivacy>'.($security ? 'Yes' : 'No').'</service:domainprivacy>
                </extension:service>
                <extension:data tld="training">
                    <extension:registry>
                        <registry:language>DE</registry:language>
                    </extension:registry>
                </extension:data>
            </extension:'.$action.'>';

            $nsUpdate = true;
        }

        // # CHECK WHEREVER IT IS ALWAYS TRUE AND FROM WHERE I GET (YES OR NO) VALUE ##

        if (!$nsUpdate) {
            $extension .= "\n";
            $extension .=
            '<extension:create>
                <extension:service>
                    <service:domainprivacy>'.($security ? 'Yes' : 'No').'</service:domainprivacy>
                </extension:service>
                ';

            if ((count(explode('.', $tld)) > 1 && 'au' === explode('.', $tld)[1]) || 'au' == $tld) {
                $extension .= '<extension:data tld="'.$tld.'">';
                $extension .=
                    '<extension:registry>
                        <registry:registrantname>'.$this->param['additionalfields']['Registrant Name'].'</registry:registrantname>
                        <registry:registrantidtype>'.$this->param['additionalfields']['Registrant ID Type'].'</registry:registrantidtype>
                        <registry:registrantid>'.$this->param['additionalfields']['Registrant ID'].'</registry:registrantid>
                        <registry:eligibilitytype>'.$this->param['additionalfields']['Eligibility Type'].'</registry:eligibilitytype>
                        <registry:eligibilityname>'.$this->param['additionalfields']['Eligibility Name'].'</registry:eligibilityname>
                        <registry:eligibilityidtype>'.$this->getEligibilityIdType($this->param['additionalfields']['Eligibility ID Type']).'</registry:eligibilityidtype>
                        <registry:eligibilityid>'.$this->param['additionalfields']['Eligibility ID'].'</registry:eligibilityid>
                        <registry:policyreason>'.$this->getPolicyReason($this->param['additionalfields']['Eligibility Reason']).'</registry:policyreason>
                    </extension:registry>';
                $extension .= '</extension:data>';
            }

            $extension .= '</extension:create>';
        }

        return $extension;
    }

    private function getPolicyReason($value)
    {
        if (false !== strpos(strtolower($value), 'trademark')) {
            return '1';
        }

        return '2';
    }

    private function getRegistrantLegalType($value)
    {
        $values = ['Corporation' => 'CCO', 'Canadian Citizen' => 'CCT', 'Permanent Resident of Canada' => 'RES', 'Government' => 'GOV', 'Canadian Educational Institution' => 'EDU',
            'Canadian Unincorporated Association' => 'ASS', 'Canadian Hospital' => 'HOP', 'Partnership Registered in Canada' => 'PRT', 'Trade-mark registered in Canada' => 'TDM',
            'Canadian Trade Union' => 'TRD', 'Canadian Political Party' => 'PLT', 'Canadian Library Archive or Museum' => 'LAM', 'Trust established in Canada' => 'TRS',
            'Aboriginal Peoples' => 'ABO', 'Legal Representative of a Canadian Citizen' => 'LGR', 'Official mark registered in Canada' => 'TDM', ];

        if (array_key_exists($value, $values)) {
            return $values[$value];
        }

        return '';
    }

    private function getEligibilityIdType($value)
    {
        if (false !== strpos(strtolower($value), 'acn')) {
            return 'ACN';
        }
        if (false !== strpos(strtolower($value), 'act')) {
            return 'ACT BN';
        }
        if (false !== strpos(strtolower($value), 'nsw')) {
            return 'NSW BN';
        }
        if (false !== strpos(strtolower($value), 'nt')) {
            return 'NT BN';
        }
        if (false !== strpos(strtolower($value), 'qld')) {
            return 'QLD BN';
        }
        if (false !== strpos(strtolower($value), 'sa')) {
            return 'SA BN';
        }
        if (false !== strpos(strtolower($value), 'tas')) {
            return 'TAS BN';
        }
        if (false !== strpos(strtolower($value), 'vic')) {
            return 'VIC BN';
        }
        if (false !== strpos(strtolower($value), 'wa')) {
            return 'WA BN';
        }
        if (false !== strpos(strtolower($value), 'tm')) {
            return 'TM';
        }
        if (false !== strpos(strtolower($value), 'other')) {
            return 'OTHER';
        }
        if (false !== strpos(strtolower($value), 'abn')) {
            return 'ABN';
        }
    }

    /**
     * FUNCTION setNs
     * Set name server from array $this->param.
     *
     * @param string $action
     *
     * @return string $ns
     */
    private function setNs($action)
    {
        $i = 0;
        $ns = null;
        foreach ($this->param['ns'] as $val) {
            if ('' != $val) {
                $ns .= '
                      <nameserver:'.$action.'> 
                        <nameserver:priority>'.$i.'</nameserver:priority> 
                        <nameserver:fqdn>'.$val.'</nameserver:fqdn> 
                        <nameserver:ipaddr>'.gethostbyname($val).'</nameserver:ipaddr> 
                      </nameserver:'.$action.'>';
            }
            ++$i;
        }

        return $ns;
    }

    /**
     * FUNCTION setContact
     * Set contact details from array $this->param.
     *
     * @param string $action
     * @param string $usetrustee
     *
     * @return string $contact
     */
    private function setContact($action, $usetrustee)
    {
        require_once dirname(__FILE__).'/countries.php';
        $ccl = new CountryCodeList();

        if (!$this->startsWith($this->param['billingphonenumber'], '+')) {
            $billingPhone = $this->getPhoneExtension($this->param['billingcountry_code'], $ccl->getCountryPhoneCodeByCountryCode($this->param['billingcountry_code']), $this->param['billingphonenumber']);
        } else {
            $billingPhone = $this->param['billingphonenumber'];
        }

        if (!$this->startsWith($this->param['phonenumber'], '+')) {
            $contactPhone = $this->getPhoneExtension($this->param['country_code'], $ccl->getCountryPhoneCodeByCountryCode($this->param['country_code']), $this->param['phonenumber']);
        } else {
            $contactPhone = $this->param['phonenumber'];
        }

        if (!$this->startsWith($this->param['adminphonenumber'], '+')) {
            $adminPhone = $this->getPhoneExtension($this->param['admincountry_code'], $ccl->getCountryPhoneCodeByCountryCode($this->param['admincountry_code']), $this->param['adminphonenumber']);
        } else {
            $adminPhone = $this->param['adminphonenumber'];
        }

        if (!$this->startsWith($this->param['techphonenumber'], '+')) {
            $techPhone = $this->getPhoneExtension($this->param['techcountry_code'], $ccl->getCountryPhoneCodeByCountryCode($this->param['techcountry_code']), $this->param['techphonenumber']);
        } else {
            $techPhone = $this->param['techphonenumber'];
        }

        $contact = '
                <contact:'.$action.'>  
                  <contact:type>billing</contact:type> 
                  <contact:usetrustee>'.$usetrustee.'</contact:usetrustee> 
                  <contact:firstname>'.$this->param['billingfirstname'].'</contact:firstname> 
                  <contact:lastname>'.$this->param['billinglastname'].'</contact:lastname> 
                  <contact:company>'.$this->param['billingcompany'].'</contact:company> 
                  <contact:address1>'.$this->param['billingaddress1'].'</contact:address1> 
                  <contact:address2>'.$this->param['billingaddress2'].'</contact:address2> 
                  <contact:city>'.$this->param['billingcity'].'</contact:city> 
                  <contact:zipcode>'.$this->param['billingpostcode'].'</contact:zipcode>
                  <contact:country_code>'.$this->param['billingcountry_code'].'</contact:country_code>
                  <contact:email>'.$this->param['billingemail'].'</contact:email> 
                  <contact:phone>'.$billingPhone.'</contact:phone> 
                  <contact:fax>'.$this->param['billinfax'].'</contact:fax> 
               </contact:'.$action.'>
               
               <contact:'.$action.'>  
                  <contact:type>org</contact:type> 
                  <contact:usetrustee>'.$usetrustee.'</contact:usetrustee> 
                  <contact:firstname>'.$this->param['firstname'].'</contact:firstname> 
                  <contact:lastname>'.$this->param['lastname'].'</contact:lastname> 
                  <contact:company>'.$this->param['company'].'</contact:company> 
                  <contact:address1>'.$this->param['address1'].'</contact:address1> 
                  <contact:address2>'.$this->param['address2'].'</contact:address2> 
                  <contact:city>'.$this->param['city'].'</contact:city> 
                  <contact:zipcode>'.$this->param['postcode'].'</contact:zipcode>
                  <contact:country_code>'.$this->param['country_code'].'</contact:country_code>
                  <contact:email>'.$this->param['email'].'</contact:email> 
                  <contact:phone>'.$contactPhone.'</contact:phone>
                  <contact:fax>'.$this->param['fax'].'</contact:fax> 
               </contact:'.$action.'> 

               <contact:'.$action.'>  
                  <contact:type>admin</contact:type> 
                  <contact:usetrustee>'.$usetrustee.'</contact:usetrustee> 
                  <contact:firstname>'.$this->param['adminfirstname'].'</contact:firstname> 
                  <contact:lastname>'.$this->param['adminlastname'].'</contact:lastname> 
                  <contact:company>'.$this->param['admincompany'].'</contact:company> 
                  <contact:address1>'.$this->param['adminaddress1'].'</contact:address1> 
                  <contact:address2>'.$this->param['adminaddress2'].'</contact:address2> 
                  <contact:city>'.$this->param['admincity'].'</contact:city> 
                  <contact:zipcode>'.$this->param['adminpostcode'].'</contact:zipcode>
                  <contact:country_code>'.$this->param['admincountry_code'].'</contact:country_code>
                  <contact:email>'.$this->param['adminemail'].'</contact:email> 
                  <contact:phone>'.$adminPhone.'</contact:phone> 
                  <contact:fax>'.$this->param['adminfax'].'</contact:fax> 
               </contact:'.$action.'> 
               
               <contact:'.$action.'>  
                  <contact:type>tech</contact:type> 
                  <contact:usetrustee>'.$usetrustee.'</contact:usetrustee> 
                  <contact:firstname>'.$this->param['techfirstname'].'</contact:firstname> 
                  <contact:lastname>'.$this->param['techlastname'].'</contact:lastname> 
                  <contact:company>'.$this->param['techcompany'].'</contact:company> 
                  <contact:address1>'.$this->param['techaddress1'].'</contact:address1> 
                  <contact:address2>'.$this->param['techaddress2'].'</contact:address2> 
                  <contact:city>'.$this->param['techcity'].'</contact:city> 
                  <contact:zipcode>'.$this->param['techpostcode'].'</contact:zipcode>
                  <contact:country_code>'.$this->param['techcountry_code'].'</contact:country_code>
                  <contact:email>'.$this->param['techemail'].'</contact:email> 
                  <contact:phone>'.$techPhone.'</contact:phone> 
                  <contact:fax>'.$this->param['techfax'].'</contact:fax> 
               </contact:'.$action.'>';

        return $contact;
    }

    /**
     * FUNCTION returnResult
     * Connects to the api returns success or error depending on the code from api.
     *
     * @param string $request
     *
     * @return string
     */
    private function returnResult($request)
    {
        // $this->request=$request;
        // $response =$this->connect($request);
        // $this->response=$response;

        return $this->connect($request);
        // $xml=$response;
        // $response= simplexml_load_string($response);
        //
        // if (!strstr($response, 'Not Found'))
        // {
        //    if (is_object($response))
        //    {
        //        if ($response->result->attributes() == 1000 || $response->result->attributes() == 1001)
        //        {
        //            return 'success';
        //        }
        //        else
        //        {
        //            $dom = new DOMDocument();
        //            $dom->loadXML($xml);
        //            $this->error = null;
        //            $msg = $dom->getElementsByTagName('msg');
//
        //            foreach ($msg as $desc)
        //            {
        //                $this->error.=$desc->textContent . "\n";
        //            }
        //        }
        //    }
        //    else
        //    {
        //        $this->error = 'Unexpected error:'. $this->response;
        //    }
        // }
    }

    /**
     * FUNCTION getCode
     * Checks answer code.
     *
     * @param string $response
     *
     * @return true, false
     */
    private function getCode($response)
    {
        $xmlDoc = new DOMDocument();
        $xmlDoc->loadXML($response);
        $result = $xmlDoc->getElementsByTagName('result')[0];

        if ($result) {
            $code = $result->getAttribute('code');

            if ('1000' !== $code && '1001' !== $code) {
                $msg = $xmlDoc->getElementsByTagName('msg');

                foreach ($msg as $desc) {
                    $this->error .= $desc->textContent."\n";
                }

                return $code;
            }

            return $code;
        }

        $this->error = 'response null on code';

        return '0';
    }

    private function GetListOfRecords($record, $isLast = false)
    {
        $rectmp = '<div '.($isLast ? 'id="dns_record_element_selectable"' : '').'  class="dns_record_element">';
        if (!$isLast) {
            $rectmp .= '<div class="dns_bin"><div class="fa fa-trash"></div><div style=" height: 60px; margin-left: 5px; margin-top: 16px;"><input type="checkbox" name="dnsrecordremove[]" value="'.$record['id'].'" /></div></div>';
        } else {
            $rectmp .= '<div class="dns_plus"><div class="fa fa-plus"></div></div>';
        }

        $rectmp .= '<div class="dns_type"><div class="dns_label_div"><label>Record Type:</label></div><div class="dns_element"><select '.($isLast ? 'onchange="dns_change(this)"' : '').' '.($isLast ? 'id="dns_selectable"' : '').'  name="dnsrecordtype[]" >
			<option value="A" '.('A' === $record['type'] ? 'selected="selected"' : '').'>A (IPv4 Address)</option>
			<option value="AAAA" '.('AAAA' === $record['type'] ? 'selected="selected"' : '').'>AAAA (IPv6 Address)</option>
			<option value="CNAME" '.('CNAME' === $record['type'] ? 'selected="selected"' : '').'>CNAME (Canonical Name)</option>
			<option value="MX" '.('MX' === $record['type'] ? 'selected="selected"' : '').'>MX (Mail Exchanger)</option>
			<option value="SPF" '.('SPF' === $record['type'] ? 'selected="selected"' : '').'>SPF (Sender Policy Framework)</option>
			<option value="SRV" '.('SRV' === $record['type'] ? 'selected="selected"' : '').'>SRV (Service Locator)</option>
			<option value="TXT" '.('TXT' === $record['type'] ? 'selected="selected"' : '').'>TXT (Text)</option>
		</select></div></div>';
        $rectmp .= '<div class="dns_host"><div class="dns_label_div"><label>Host name:</label></div><div class="dns_element"><input type="text" name="dnsrecordhost[]" value="'.$record['host'].'" class="input-small" /></div></div>';

        switch ($record['type']) {
            case 'SRV':
            $rectmp .= '<div class="dns_data"><div class="dns_label_div"><label>Service:</label></div><div class="dns_element"><input type="text" name="dnsrecordaddress[]" value="'.$record['data'].'"  class="input-small" /></div></div>';
                $rectmp .= '<div class="dns_data"><div class="dns_label_div"><label>weight:</label></div><div class="dns_element"><input type="number" name="dnsrecordsrvweight[]" value="'.$record['weight'].'"  min="10" max="100" step="10" class="input-small" /></div></div>';
                $rectmp .= '<div class="dns_ttl"><div class="dns_label_div"><label>Port:</label></div><div class="dns_element"><input type="number" name="dnsrecordsrvport[]" value="'.$record['port'].'"  class="input-small" /></div></div>';
                $rectmp .= '<div class="dns_data"><div class="dns_label_div"><label>Priority:</label></div><div class="dns_element"><input type="number" name="dnsrecordsrvpriority[]" value="'.$record['priority'].'"  min="10" max="100" step="10" class="input-small" /></div></div>';

                break;

            case 'A':
                $rectmp .= '<div class="dns_data"><div class="dns_label_div"><label>Address IPV4:</label></div><div class="dns_element"><input type="text" name="dnsrecordaddress[]" value="'.$record['data'].'" class="input-small" /></div></div>';

                break;

            case 'AAAA':
                $rectmp .= '<div class="dns_data"><div class="dns_label_div"><label>Address IPV6:</label></div><div class="dns_element"><input type="text" name="dnsrecordaddress[]" value="'.$record['data'].'" class="input-small" /></div></div>';

                break;

            case 'CNAME':
                $rectmp .= '<div class="dns_data"><div class="dns_label_div"><label>Canonical Name:</label></div><div class="dns_element"><input type="text" name="dnsrecordaddress[]" value="'.$record['data'].'" class="input-small" /></div></div>';

                break;

            case 'NS':
                $rectmp .= '<div class="dns_data"><div class="dns_label_div"><label>Name Server:</label></div><div class="dns_element"><input type="text" name="dnsrecordaddress[]" value="'.$record['data'].'" class="input-small" /></div></div>';

                break;

            case 'SPF':
                $rectmp .= '<div class="dns_data"><div class="dns_label_div"><label>Rule:</label></div><div class="dns_element"><input type="text" name="dnsrecordaddress[]" value="'.$record['data'].'"  class="input-small" /></div></div>';

                break;

            case 'TXT':
                $rectmp .= '<div class="dns_data"><div class="dns_label_div"><label>Text:</label></div><div class="dns_element"><input type="text" name="dnsrecordaddress[]" value="'.$record['data'].'"  class="input-small" /></div></div>';

                break;

            case 'MX':
                $rectmp .= '<div class="dns_data"><div class="dns_label_div"><label>Mail Server:</label></div><div class="dns_element"><input type="text" name="dnsrecordaddress[]" value="'.$record['data'].'"  class="input-small" /></div></div>';
                $rectmp .= '<div class="dns_data"><div class="dns_label_div"><label>Priority:</label></div><div class="dns_element"><input type="number" name="dnsrecordmxpriority[]" value="'.$record['mx_priority'].'"  min="10" max="100" step="10" class="input-small" /></div></div>';

                break;
        }

        // /TTL
        $rectmp .= '<div class="dns_ttl"><div class="dns_label_div"><label >TTL in sec:</label></div><div class="dns_element">
		<select name="dnsrecordttl[]" >
			<option value="600" '.('600' === $record['ttl'] ? 'selected="selected"' : '').'>10 min</option>
			<option value="900" '.('900' === $record['ttl'] ? 'selected="selected"' : '').'>15 min</option>
			<option value="1800" '.('1800' === $record['ttl'] ? 'selected="selected"' : '').'>30 min</option>
			<option value="3600" '.('3600' === $record['ttl'] ? 'selected="selected"' : '').'>1h</option>
			<option value="7200" '.('7200' === $record['ttl'] ? 'selected="selected"' : '').'>2h</option>
			<option value="14400" '.('14400' === $record['ttl'] ? 'selected="selected"' : '').'>4h</option>
			<option value="21600" '.('21600' === $record['ttl'] ? 'selected="selected"' : '').'>6h</option>
			<option value="43200" '.('43200' === $record['ttl'] ? 'selected="selected"' : '').'>12h</option>
			<option value="86400" '.('86400' === $record['ttl'] ? 'selected="selected"' : '').'>24h</option>
			<option value="172800" '.('172800' === $record['ttl'] ? 'selected="selected"' : '').'>2d</option>
			<option value="432000" '.('432000' === $record['ttl'] ? 'selected="selected"' : '').'>5d</option>
			<option value="604800" '.('604800' === $record['ttl'] ? 'selected="selected"' : '').'>7d</option>
			</select></div><input type="hidden" name="dnsrecordrecid[]" value="'.$record['id'].'" /></div>';

        return $rectmp.'</div>';
    }

    private function GetJSReplaceCode($html, $formUrl)
    {
        $htmltmp = preg_replace('/\s+/', ' ', trim($html));
        $jsOutput = '<script>
		var old_form;
		var div = document.createElement("div");
		';
        $jsOutput .= 'var htmlformoutput = \''.$htmltmp.'\';
		var ATemp = \''.preg_replace('/\s+/', ' ', trim($this->GetListOfRecords(['host' => '', 'type' => 'A', 'data' => '', 'ttl' => '3600'], true))).'\';
		var AAAATemp = \''.preg_replace('/\s+/', ' ', trim($this->GetListOfRecords(['host' => '', 'type' => 'AAAA', 'data' => '', 'ttl' => '3600'], true))).'\';
		var CNAMETemp = \''.preg_replace('/\s+/', ' ', trim($this->GetListOfRecords(['host' => '', 'type' => 'CNAME', 'data' => '', 'ttl' => '3600'], true))).'\';
		var MXTemp = \''.preg_replace('/\s+/', ' ', trim($this->GetListOfRecords(['host' => '', 'type' => 'MX', 'data' => '', 'ttl' => '3600', 'mx_priority' => '10'], true))).'\';
		var NSTemp = \''.preg_replace('/\s+/', ' ', trim($this->GetListOfRecords(['host' => '', 'type' => 'NS', 'data' => '', 'ttl' => '3600'], true))).'\';
		var SPFTemp = \''.preg_replace('/\s+/', ' ', trim($this->GetListOfRecords(['host' => '', 'type' => 'SPF', 'data' => '', 'ttl' => '3600'], true))).'\';
		var SRVTemp = \''.preg_replace('/\s+/', ' ', trim($this->GetListOfRecords(['host' => '', 'type' => 'SRV', 'data' => '', 'ttl' => '3600', 'weight' => '10', 'priority' => '10', 'port' => ''], true))).'\';
		var TXTTemp = \''.preg_replace('/\s+/', ' ', trim($this->GetListOfRecords(['host' => '', 'type' => 'TXT', 'data' => '', 'ttl' => '3600'], true))).'\';';
        $jsOutput .= '
		div.innerHTML = htmlformoutput;
		document.addEventListener("DOMContentLoaded", function() {
     old_form = jQuery( "form[action=\''.$formUrl.'\']" )[0];
	 old_form.parentElement.append(div);
	 old_form.parentElement.removeChild(old_form);
	
}, false);
function dns_change(e)
{
	switch(e.options[e.selectedIndex].value) 
	{
		case "A":
			jQuery("#dns_record_element_selectable").remove();
			div = document.createElement("div");
			div.innerHTML = ATemp;
			jQuery("#dns_record_table").append(div);
			break;
		case "AAAA":
			jQuery("#dns_record_element_selectable").remove();
			div = document.createElement("div");
			div.innerHTML = AAAATemp;
			jQuery("#dns_record_table").append(div);
			break;
		case "CNAME":
			jQuery("#dns_record_element_selectable").remove();
			div = document.createElement("div");
			div.innerHTML = CNAMETemp;
			jQuery("#dns_record_table").append(div);
			break;
		case "MX":
			jQuery("#dns_record_element_selectable").remove();
			div = document.createElement("div");
			div.innerHTML = MXTemp;
			jQuery("#dns_record_table").append(div);
			break;
		case "SPF":
			jQuery("#dns_record_element_selectable").remove();
			div = document.createElement("div");
			div.innerHTML = SPFTemp;
			jQuery("#dns_record_table").append(div);
			break;
		case "TXT":
			jQuery("#dns_record_element_selectable").remove();
			div = document.createElement("div");
			div.innerHTML = TXTTemp;
			jQuery("#dns_record_table").append(div);
			break;
		case "NS":
			jQuery("#dns_record_element_selectable").remove();
			div = document.createElement("div");
			div.innerHTML = NSTemp;
			jQuery("#dns_record_table").append(div);
			break;
		case "SRV":
			jQuery("#dns_record_element_selectable").remove();
			div = document.createElement("div");
			div.innerHTML = SRVTemp;
			jQuery("#dns_record_table").append(div);
			break;
		default:
			
	}
}
';
        $jsOutput .= '';
        $jsOutput .= '</script>';
        $jsOutput .= $this->GenerateCss();

        return $jsOutput;
    }

    private function GenerateCss()
    {
        $css = '<style>
		
		.dns_record_element div {
			float: left;
			
		}
		.dns_element {
			float: left;
			margin: 20px 5px 5px 5px;
		}
		
		.dns_record_element {
			width: 100%;
			margin-bottom: 15px;
			float: left;
			border: 1px solid #ddd;
			background-color: lightyellow;
		}
		.dns_type div select {
			width: 230px;
		}
		
		.dns_label_div {
			position: absolute;
			margin-left:21px;
		}
		
		.dns_record_element select,.dns_record_element input
		{
			height: 34px;
			padding: 6px 12px;
			font-size: 14px;
			border: 1px solid #ccc;
			border-radius: 4px;
			font-size: 13px;
		}
		
		.fa-trash
		{
			position: absolute;
			margin-left: 6px;
			margin-top: 5px;
		}
		
		.dns_plus
		{
			margin-left: 6px;
			margin-top: 30px;
		}
		#dns_info_message
		{
			color: red;
			margin-bottom: 5px;
		}
		
		';

        $css .= '</style>';

        return $css;
    }

    private function getPhoneExtension($code, $countryCodeArray, $phoneNumber)
    {
        $res = $phoneNumber;
        if ($this->startsWith($res, '00'.$countryCodeArray['code'])) {
            $length = strlen('00'.$countryCodeArray['code']);

            return '+'.$countryCodeArray['code'].substr($res, $length);
        }

        if ('DE' === $code) {
            if ($this->startsWith($res, '0')) {
                return '+'.$countryCodeArray['code'].substr($res, 1);
            }

            return '+'.$countryCodeArray['code'].$res;
        }

        return '+'.$countryCodeArray['code'].$res;
    }
}
